Package: henna
Type: Package
Title: A Versatile Visualization Suite
Version: 0.8.5
Authors@R: person('Andrei-Florian', 'Stoica', 
      email='andreistoica@foxmail.com',
     role=c('aut','cre'),
     comment=c(ORCID='0000-0002-5253-0826'))
Description: A visualization suite primarily designed for single-cell
    RNA-sequencing data analysis applications but well-suited for
    other purposes as well. It introduces novel plots to represent two-variable
    and frequency data and optimizes some commonly used plotting options
    (e.g., correlation, network, density, alluvial and volcano plots) 
    for ease of usage and flexibility.
License: MIT + file LICENSE
Imports: abdiv, dplyr, ggalluvial, ggeasy, ggforce, ggraph, ggnewscale,
        ggplot2, ggrepel, grDevices, liver, methods, paletteer,
        reshape2, rlang, stats, tidygraph, viridis, withr
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: EnhancedVolcano, qs2, testthat (>= 3.0.0)
URL: https://github.com/andrei-stoica26/henna
BugReports: https://github.com/andrei-stoica26/henna/issues
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-07-13 02:39:38 UTC; Andrei
Author: Andrei-Florian Stoica [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5253-0826>)
Maintainer: Andrei-Florian Stoica <andreistoica@foxmail.com>
Repository: CRAN
Date/Publication: 2026-07-13 05:40:02 UTC

# Generated by roxygen2: do not edit by hand

export(centerTitle)
export(classPlot)
export(connectedComponents)
export(convexHull)
export(correlationPlot)
export(densityPlot)
export(dpColors)
export(hpColors)
export(hullPlot)
export(isPointOnBoundary)
export(isPointOnSeg)
export(labelPoints)
export(networkPlot)
export(pointsToSegments)
export(radialPlot)
export(rankPlot)
export(rankSummary)
export(reorderDF)
export(riverPlot)
export(rpColors)
export(tilePlot)
export(vertexComponents)
export(volcanoPlot)
importFrom(abdiv,euclidean)
importFrom(dplyr,count)
importFrom(dplyr,mutate)
importFrom(ggalluvial,StatStratum)
importFrom(ggalluvial,geom_alluvium)
importFrom(ggalluvial,geom_stratum)
importFrom(ggeasy,easy_remove_axes)
importFrom(ggforce,geom_circle)
importFrom(ggnewscale,new_scale_color)
importFrom(ggnewscale,new_scale_fill)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_label)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_color_discrete)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_fill_viridis_d)
importFrom(ggplot2,stat_density_2d)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_linedraw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,xlab)
importFrom(ggraph,geom_edge_link)
importFrom(ggraph,geom_node_point)
importFrom(ggraph,geom_node_text)
importFrom(ggraph,ggraph)
importFrom(ggraph,scale_edge_width)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,chull)
importFrom(grDevices,hcl.colors)
importFrom(liver,minmax)
importFrom(methods,is)
importFrom(paletteer,paletteer_c)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,density)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(tidygraph,activate)
importFrom(tidygraph,as_tbl_graph)
importFrom(viridis,scale_color_viridis)
importFrom(viridis,scale_fill_viridis)
importFrom(withr,with_seed)

# henna
`henna` is a suite comprising several visualization utilities. 
They are designed with scRNA-seq data analysis in mind but are well-suited
to a variety of other applications as well.

## Installation

To install the version of `henna` currently available on CRAN, run the 
following R code:

```r
install.packages("henna")
```

Alternatively, you can install the most recent development version 
using this code: 

```r
pak::pak("andrei-stoica26/henna")
```

Currently, both versions are the same (`0.8.5`).

## Visualization tools

The data used to generate the figures presented here comes from the human Baron 
pancreas (HBP) dataset, which can be accessed through the [scRNAseq](https://bioconductor.org/packages/release/data/experiment/html/scRNAseq.html)
package.  The code used to generate the figures is provided in the 
[scripts](https://github.com/andrei-stoica26/henna/tree/main/inst/scripts) 
folder.

### classPlot

This function creates a bar plot that quantifies items belonging
to different classes. Its required input is a data frame whose first two
columns are categorical (classes and items) and third column is numeric
(counts).

![](inst/figures/class_plot.png)

### correlationPlot

This function plots a correlation matrix. It accepts a matrix or a data frame
as input.

![](inst/figures/correlation_plot.png)

### densityPlot

This function creates a density plot for a set of points, optionally displaying
segments between each point and its nearest neighbor. It requires a data frame
with the `x` and `y` coordinates of the points, which may additionally
contain score and nearest-neighbor columns.

![](inst/figures/density_plot.png)

### hullPlot

This function plots a set of points together with their convex hull. 
Optionally, it divides the points into 2 or 4 classes of different colors by 
drawing a vertical and/or a horizontal line that intersects the hull. It can
also label selected points. The required input is a data frame with `x` 
and `y` coordinates.

![](inst/figures/hull_plot.png)

### networkPlot

This function plots a graph using different colors for nodes that are part of 
different connected components. It requires as input a data frame whose first
two columns are categorical and represent the edges in a graph.
A numeric column representing either edge weights or edge ranks (these will
be converted into weights) can also be provided. If available, this column will
be used to vary edge width.

![](inst/figures/network_plot.png)

### radialPlot

This function plots positive numeric-valued points over concentric circles, 
with points located more centrally representing higher values. Optionally, the
points can be assigned to groups and colored based on these groups. 

The input data frame must store the point labels in its first column and their
numeric values in its second column. A third column (categorical) representing
the group where each point is assigned can also be provided.

![](inst/figures/radial_plot.png)

### rankPlot
This function aggregates multiple ranks available for a group of items in a 
single plot. Mean ranks can be displayed on the plot.

![](inst/figures/rank_plot.png)

### riverPlot
This function generates an alluvial plot for a data frame with two categorical 
columns and a numeric column. In the example below, it represents the 
significance of pairs of overlaps between the markers of groups belonging to 
two identity classes (cell type and donor). 

![](inst/figures/river_plot.png)

Stronger overlaps correspond to thicker connecting curves.

### tilePlot

This function plots a numeric matrix. It is a more general version of
`correlationPlot`.

![](inst/figures/tile_plot.png)

### volcanoPlot

If the Bioconductor dependency [EnhancedVolcano](https://bioconductor.org/packages/release/bioc/html/EnhancedVolcano.html)
(not installed automatically with `henna`) is available, this function can be used
to create a volcano plot for a data frame with a log column and a p-value 
column. The gene names must be provided as row names.

Users can input labeled genes in two ways: 

1. By using p-value and log fold-change thresholds (the default option). 
2. By inputting a list of labels. 

The default option is preferred in the example below. To emphasize the usage 
of different thresholds for labeling and significance, the thresholds for
labeling are set much higher than the thresholds for significance.

![](inst/figures/volcano_plot.png)

# A self-contained script to generate the data in inst/extdata
#'
createExtData <- function(){
    if (requireNamespace(c('CSOA', 'dplyr', 'hammers', 'LISTO', 'qs2',
                           'scRNAseq','scuttle','Seurat'), quietly=TRUE)){
        scObj <- scRNAseq::BaronPancreasData('human')
        scObj <- scuttle::logNormCounts(scObj)
        scObj <- Seurat::as.Seurat(scObj)

        df <- dplyr::count(scObj[[]], donor, label)
        qs2::qs_save(df, 'inst/extdata/classPlot.qs2')

        genes <- sort(c('FLNA','SERPING1', 'SERPINH1', 'CHGA', 'CHGB','COL1A1',
                        'COL6A1', 'CELA3B', 'CELA3A', 'CD24', 'KRT8', 'KRT18',
                        'NOTCH3'))
        mat <- hammers::scExpMat(scObj, genes=genes)
        corMat <- cor(t(mat))
        qs2::qs_save(corMat, 'inst/extdata/correlationPlot.qs2')

        genes <- c('COX7B', 'SMURF2', 'MINOS1','C19orf43', 'EIF3D', 'PTGES3',
                   'TMED4', 'ARL2', 'C12orf75', 'NUCKS1', 'RNF187',
                   'SMPD1', 'ACTR1A', 'TCEAL4', 'EID2B',
                   'COPRS', 'CDC42BPB', 'PDAP1', 'PRPF8', 'ATRAID', 'ILF3',
                   'CERS2', 'TBCK', 'FNDC3A', 'VAPB', 'AKAP9')
        mat <- hammers::scExpMat(scObj, genes=genes)
        d <- stats::dist(mat, 'euclidean')
        df <- data.frame(cmdscale(d))
        qs2::qs_save(df, 'inst/extdata/densityPlot.qs2')

        markers <- LISTO::buildMarkerList(scObj, 'label',
                                          ids1=c('alpha', 'beta',
                                                 'gamma', 'delta'))
        sharedDF <- LISTO::sharedMarkers(markers[[1]], markers[[4]])
        qs2::qs_save(sharedDF, 'inst/extdata/hullPlot.qs2')

        alphaMarkers <- c('GCG', 'TTR', 'PCSK2', 'FXYD5', 'LDB2', 'MAFB',
                          'CHGA', 'SCGB2A1', 'GLS', 'FAP', 'DPP4', 'GPR119',
                          'PAX6', 'NEUROD1', 'LOXL4', 'PLCE1', 'GC', 'KLHL41',
                          'FEV', 'PTGER3', 'RFX6', 'SMARCA1', 'PGR', 'IRX1',
                          'UCP2', 'RGS4', 'KCNK16', 'GLP1R', 'ARX', 'POU3F4',
                          'RESP18', 'PYY', 'SLC38A5', 'TM4SF4', 'CRYBA2', 'SH3GL2',
                          'PCSK1', 'PRRG2', 'IRX2', 'ALDH1A1','PEMT', 'SMIM24',
                          'F10', 'SCGN', 'SLC30A8')

        deltaMarkers <- c('SST', 'GABRB3', 'FRZB', 'MS4A8', 'BAIAP3', 'CASR', 'BCHE',
                          'UNC5B','EDN3', 'GHSR', 'PCSK1', 'GABRG2', 'POU3F1',
                          'BHLHE41', 'EHF', 'LCORL', 'ETV1', 'PDX1', 'LEPR', 'UCP2',
                          'NPTX2', 'FXYD2', 'IAPP', 'KCNK16', 'SCGN', 'ISL1', 'HHEX',
                          'RESP18', 'PAX4', 'RBP4', 'PCSK9', 'FFAR4')


        ductalMarkers <-  c('CFTR', 'SERPINA5', 'SLPI', 'TFF1', 'CFB', 'LGALS4',
                            'CTSH',	'PERP', 'PDLIM3',	'WFDC2', 'SLC3A1', 'AQP1',
                            'ALDH1A3', 'VTCN1',	'KRT19', 'TFF2', 'KRT7', 'CLDN4',
                            'LAMB3', 'TACSTD2', 'CCL2', 'DCDC2','CXCL2', 'CLDN10',
                            'HNF1B', 'KRT20', 'MUC1', 'ONECUT1', 'AMBP', 'HHEX',
                            'ANXA4', 'SPP1', 'PDX1', 'SERPINA3', 'GDF15', 'AKR1C3',
                            'MMP7', 'DEFB1', 'SERPING1', 'TSPAN8', 'CLDN1', 'S100A10',
                            'PIGR')

        gammaMarkers <- c('PPY', 'ABCC9', 'FGB', 'ZNF503', 'MEIS1', 'LMO3', 'EGR3',
                          'CHN2', 'PTGFR', 'ENTPD2', 'AQP3', 'THSD7A', 'CARTPT',
                          'ISL1', 'PAX6', 'NEUROD1', 'APOBEC2', 'SEMA3E', 'SLITRK6',
                          'SERTM1', 'PXK', 'PPY2P', 'ETV1', 'ARX', 'CMTM8', 'SCGB2A1',
                          'FXYD2', 'SCGN')

        genes <- Reduce(union, list(alphaMarkers, deltaMarkers,
                                    ductalMarkers, gammaMarkers))

        geneSetExp <- hammers::scExpMat(scObj, genes=genes)
        overlapDF <- CSOA::generateOverlaps(geneSetExp)
        overlapDF <- CSOA::processOverlaps(overlapDF)
        qs2::qs_save(overlapDF, 'inst/extdata/networkPlot.qs2')

        overlapObj <- CSOA:::edgeLists.data.frame(overlapDF)
        degreesDF <- CSOA:::geneDegrees(overlapObj)
        qs2::qs_save(degreesDF, 'inst/extdata/radialPlot.qs2')

        shared <- Reduce(intersect, list(
            rownames(markers[['alpha']]), rownames(markers[['beta']]),
            rownames(markers[['delta']]), rownames(markers[['gamma']])
        ))[seq(12)]

        logsDF <- data.frame(alphaLog = markers[['alpha']][shared, ]$avg_log2FC,
                             betaLog = markers[['beta']][shared, ]$avg_log2FC,
                             deltaLog = markers[['delta']][shared, ]$avg_log2FC,
                             gammaLog = markers[['gamma']][shared, ]$avg_log2FC)
        rownames(logsDF) <- shared
        ranksDF <- apply(logsDF, 2, function(x) rank(-x, ties.method='min'))
        qs2::qs_save(ranksDF, 'inst/extdata/rankPlot.qs2')

        donorMarkers <- LISTO::buildMarkerList(scObj, 'donor')
        markerOverlaps <- LISTO::markerDFListOverlap(markers,
                                                     donorMarkers,
                                                     rownames(scObj))
        riverDF <- hammers::prepAlluvial(markerOverlaps)
        qs2::qs_save(riverDF, 'inst/extdata/riverPlot.qs2')

        celltypes <- names(markers)
        mat <- do.call(rbind, lapply(celltypes, function(i){
            sapply(celltypes, function(j)
                length(intersect(rownames(markers[[i]]),
                                 rownames(markers[[j]]))))
        }))
        rownames(mat) <- celltypes
        colnames(mat) <- celltypes
        qs2::qs_save(mat, 'inst/extdata/tilePlot.qs2')

        df <- Seurat::FindMarkers(scObj,
                                  group.by='label',
                                  ident.1='beta',
                                  logfc.threshold=0,
                                  min.pct=0)
        df <- df[df$p_val_adj < 0.05, ]
        qs2::qs_save(df, 'inst/extdata/volcanoPlot.qs2')

    }
}

createExtData()

# Script to generate the figures in man/figures
#'
createPlot1 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'),
                         quietly=TRUE)){
        df <- qs2::qs_read('inst/extdata/classPlot.qs2')
        p <- henna::classPlot(df,
                              title='Most abundant cell types grouped by donor',
                              xLab='Count',
                              yLab='Cell type',
                              legendTitle='Donor',
                              labelSize=2.1,
                              valueCutoff=100)
        ggplot2::ggsave("inst/figures/class_plot.png", p, width=6, height=4)
    }
}

createPlot2 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'),
                         quietly=TRUE)){
        corMat <- qs2::qs_read('inst/extdata/correlationPlot.qs2')
        p <- henna::correlationPlot(corMat, labelSize=2.5)
        ggplot2::ggsave("inst/figures/correlation_plot.png", p, width=6,
                        height=4)
    }
}

createPlot3 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'),
                         quietly=TRUE)){
        df <- qs2::qs_read('inst/extdata/densityPlot.qs2')
        p <- henna::densityPlot(df, 'Density plot', colorScheme='sea',
                                labelSize=2)
        ggplot2::ggsave("inst/figures/density_plot.png", p, width=6,
                        height=4)
    }
}

createPlot4 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'),
                         quietly=TRUE)){
        sharedDF <- qs2::qs_read('inst/extdata/hullPlot.qs2')
        name1 <- 'alpha'
        name2 <- 'delta'
        legendLabs <- as.factor(c('Non-top',
                               'Shared',
                               paste0('Top only for ', name2),
                               paste0('Top only for ', name1)))
        p <- henna::hullPlot(sharedDF,
                             'Shared markers plot',
                             xInt=1.5,
                             yInt=1.3,
                             hullWidth=0.5,
                             labXThr=1.5,
                             labYThr=1.3,
                             xLab=paste0('avg_log2FC (', name1, ')'),
                             yLab=paste0('avg_log2FC (', name2, ')'),
                             legendLabs=legendLabs)
        ggplot2::ggsave("inst/figures/hull_plot.png", p, width=6,
                        height=4)
    }
}

createPlot5 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'), quietly=TRUE)){
        overlapDF <- qs2::qs_read('inst/extdata/networkPlot.qs2')
        p <- henna::networkPlot(overlapDF,
                                'Genes with strongly overlapping expression',
                                'rank',
                                'ranks')
        ggplot2::ggsave("inst/figures/network_plot.png", p, width=6,
                        height=4)
    }
}

createPlot6 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'), quietly=TRUE)){
        degreesDF <- qs2::qs_read('inst/extdata/radialPlot.qs2')
        p <- henna::radialPlot(degreesDF,
                               title='Genes involved in top overlaps',
                               groupLegendTitle='Component',
                               valueLegendTitle='Degree',
                               extraCircles=2)
        ggplot2::ggsave("inst/figures/radial_plot.png", p, width=6,
                        height=4)
    }
}

createPlot7 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'), quietly=TRUE)){
        ranksDF <- qs2::qs_read('inst/extdata/rankPlot.qs2')
        p <- henna::rankPlot(ranksDF,
                             paste0('Marker ranks for alpha, beta, delta ',
                                    'and gamma cells'),
                             sigDigits=2,
                             xLab='Gene')
        ggplot2::ggsave("inst/figures/rank_plot.png", p, width=6,
                        height=4)
    }
}

createPlot8 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'), quietly=TRUE)){
        riverDF <- qs2::qs_read('inst/extdata/riverPlot.qs2')
        p <- henna::riverPlot(riverDF,
                              paste0('Overlaps between cell type markers ',
                                     'and donor markers'))
        ggplot2::ggsave("inst/figures/river_plot.png", p, width=6,
                        height=4)
    }
}

createPlot9 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'), quietly=TRUE)){
        mat <- qs2::qs_read('inst/extdata/tilePlot.qs2')
        p <- henna::tilePlot(mat,
                             title='Number of shared markers',
                             xLab=NULL,
                             yLab=NULL)
        ggplot2::ggsave("inst/figures/tile_plot.png", p, width=6, height=4)
    }
}

createPlot10 <- function(){
    if (requireNamespace(c('ggplot2', 'henna', 'qs2'), quietly=TRUE)){
        df <- qs2::qs_read('inst/extdata/volcanoPlot.qs2')
        p <- henna::volcanoPlot(df,
                                title='Volcano plot - beta cells',
                                pvalThr=1e-10,
                                logFCThr=1,
                                labPvalThr=1e-150,
                                labLogFCThr=5.3)
        ggplot2::ggsave("inst/figures/volcano_plot.png", p, width=6, height=4)
    }
}

createFigures <- function()
    for (i in seq(10))
        eval(as.name(paste0('createPlot', i)))()

createFigures()

#' Create a label data frame
#'
#' This function creates a label data frame to be used by plotting functions
#' that label selected points.
#'
#' @param df A data frame.
#' @param xCol Column representing the \code{x} axis.
#' @param yCol Column representing the \code{y} axis.
#' @param labeledPoints Point labels to be displayed on the plot. Default is
#' \code{NULL}, entailing that point labels will be displayed on the basis
#' of \code{labXThr} and \code{labYThr}, if at least one of them is not
#' \code{NULL}.
#' @param labelOutside Display labels for points specified by
#' \code{labeledPoints} even if they fall outside the boundaries imposed by
#' \code{labXThr} and \code{labYThr}. Ignored if \code{labeledPoints} is
#' \code{NULL} or both \code{labXThr} and \code{labYThr} are \code{NULL}.
#' @param labXThr Threshold used to plot labels based on \code{xCol} values.
#' @param labYThr Threshold used to plot labels based on \code{yCol} values.
#' @param adjFunX Function used to adjust \code{xCol} values. Default is
#' \code{identity} (no adjustment will be performed).
#' @param adjFunY Function used to adjust \code{yCol} values. Default is
#' \code{identity} (no adjustment will be performed).
#' @param compFunX Function used for the selection of \code{xCol} values
#' based on \code{labXThr}.
#' @param compFunY Function used for the selection of \code{yCol} values
#' based on \code{labYThr}.
#'
#' @return A label data frame.
#'
#' @keywords internal
#'
createLabelDFTemplate <- function(df,
                                  xCol = 1,
                                  yCol = 2,
                                  labeledPoints = NULL,
                                  labelOutside = FALSE,
                                  labXThr = NULL,
                                  labYThr = NULL,
                                  adjFunX = identity,
                                  adjFunY = identity,
                                  compFunX = `>=`,
                                  compFunY = `>=`){
    if (is.null(labXThr) & is.null(labYThr) & is.null(labeledPoints))
        return(NULL)

    if (is.null(labXThr) & is.null(labYThr))
        return(df[labeledPoints, ])

    xyDF <- df[, c(xCol, yCol)]
    if (!is.null(labXThr))
        xyDF <- xyDF[compFunX(adjFunX(xyDF[, 1]), labXThr), ]
    if (!is.null(labYThr))
        xyDF <- xyDF[compFunY(adjFunY(xyDF[, 2]), labYThr), ]

    if(is.null(labeledPoints) | !labelOutside)
        return(xyDF)

    return(df[union(rownames(xyDF), labeledPoints), ])
}

#' Create a label data frame to be used by \code{hullPlot}.
#'
#' This function creates a label data frame to be used by \code{hullPlot}.
#'
#' @inheritParams createLabelDFTemplate
#'
#' @return A label data frame to be used by \code{hullPlot}.
#'
#' @noRd
#'
createLabelDFHull <- function(df,
                              xCol = 1,
                              yCol = 2,
                              labeledPoints = NULL,
                              labelOutside = FALSE,
                              labXThr = NULL,
                              labYThr = NULL)
    return(createLabelDFTemplate(df, xCol, yCol, labeledPoints, labelOutside,
                                 labXThr, labYThr))

#' Create a label data frame to be used by \code{hullPlot}.
#'
#' This function creates a label data frame to be used by \code{hullPlot}.
#'
#' @inheritParams createLabelDFTemplate
#'
#' @return A label data frame to be used by \code{hullPlot}.
#'
#' @noRd
#'
createLabelDFVolcano <- function(df,
                                 xCol = 2,
                                 yCol = 5,
                                 labeledPoints = NULL,
                                 labelOutside = FALSE,
                                 labXThr = NULL,
                                 labYThr = NULL){

    labelDF <- createLabelDFTemplate(df, xCol, yCol, labeledPoints,
                                     labelOutside, labXThr, labYThr,
                                     adjFunX=abs, compFunY=`<`)

    df[, yCol][df[, yCol] == 0] <- 2
    labelDF[, 2][labelDF[, 2] == 0] <- min(df[, yCol]) / 10

    labelDF[, 2] <- -log(labelDF[, 2], 10)
    return(labelDF)
}

#' Label points in a ggplot object
#'
#' This function labels points in a ggplot object.
#'
#' @inheritParams centerTitle
#' @inheritParams documentFun
#' @param labelDF Label data frame.
#' @param pointLabs Labels of points.
#' @param ... Additional arguments passed to \code{geom_text_repel}
#' (if \code{labelType} is 'free') or \code{geom_text_label}
#' (if \code{labelType} is 'boxed').
#'
#' @return A ggplot object.
#'
#' @examples
#' filePath <- system.file('extdata', 'hullPlot.qs2', package='henna')
#' sharedDF <- qs2::qs_read(filePath)
#' name1 <- 'alpha'
#' name2 <- 'delta'
#' legendLabs <- as.factor(c('Non-top',
#' 'Shared',
#' paste0('Top only for ', name2),
#' paste0('Top only for ', name1)))
#' p <- hullPlot(sharedDF, 'Shared markers plot', xInt=1.5, yInt=1.3,
#' xLab=paste0('avg_log2FC (', name1, ')'),
#' yLab=paste0('avg_log2FC (', name2, ')'),
#' legendLabs=legendLabs)
#' labelDF <- sharedDF[sharedDF[, 'avg_log2FC_1'] > 1.5 &
#' sharedDF[, 'avg_log2FC_2'] > 1.3, ]
#' p <- labelPoints(p, labelDF, labelType='boxed', nudge_x=0.1, nudge_y=0.1)
#'
#' @export
#'
labelPoints <- function(p,
                        labelDF,
                        pointLabs = rownames(labelDF),
                        labelType = c('free', 'boxed'),
                        labelSize = 2.5,
                        labelColor = 'black',
                        labelRepulsion = 1,
                        labelPull = 1,
                        maxOverlaps = 50,
                        boxPadding = 0.2,
                        labelPadding = 0.1,
                        labelSegWidth = 0.4,
                        ...){

    labelType <- match.arg(labelType, c('free', 'boxed'))
    plotFuns <- setNames(list(geom_text_repel, geom_label_repel),
                         c('free', 'boxed'))
    basicArgs <- list(mapping=aes(x=labelDF[, 1],
                                  y=labelDF[, 2],
                                  label=pointLabs),
                      data=labelDF,
                      size=labelSize,
                      color=labelColor,
                      force=labelRepulsion,
                      force_pull=labelPull,
                      max.overlaps=maxOverlaps,
                      box.padding=boxPadding,
                      segment.size=labelSegWidth)
    extraArgs <- list(...)
    if(labelType == 'boxed')
        extraArgs <- c(label.padding=labelPadding,
                       extraArgs)
    allArgs <- c(basicArgs, extraArgs)
    p <- p + do.call(plotFuns[[labelType]], allArgs)
    return(p)
}

#' @importFrom dplyr mutate
#' @importFrom grDevices chull hcl.colors
#' @importFrom ggraph geom_edge_link geom_node_point geom_node_text ggraph scale_edge_width
#' @importFrom paletteer paletteer_c
#' @importFrom tidygraph activate as_tbl_graph
#'
NULL

#' Prepare data frame for network plot
#'
#' This function prepares a data frame for network plot.
#'
#' @param df Data frame.
#' @param rankCol Name of the rank column.
#'
#' @return A data frame ready to serve as input to \code{networkPlot}.
#'
#' @keywords internal
#'
networkPlotDF <- function(df,
                          rankCol = 'rank'){
    preWeight <- log(max(df[[rankCol]]) /
                         df[[rankCol]] + 0.01)
    df$weight <- preWeight / max(preWeight)
    return(df)
}

#' Plot graph with the option of using different colors for connected components
#'
#' This function plots the graph of the data frame and optionally uses different
#' colors for nodes belonging to different connected components.
#'
#' @inheritParams documentFun
#' @inheritParams networkPlotDF
#' @param numCol Name of the numeric column used to vary edge widths. If no
#' such column is provided, set to \code{NULL}.
#' @param numColType The type of the numeric column used to vary edge widths.
#' Choose between 'weights' and 'ranks' Ignored if \code{numCol}
#' is \code{NULL}.
#' @param nodeSize Size of graph nodes.
#' @param nodeTextSize Size of text on graph nodes.
#' @param palette grDevices palette used for coloring nodes.
#' Ignored if nodeColor is not \code{NULL}.
#' @param nodeColor Color used for nodes. Default is \code{NULL}
#' (\code{palette} will instead be used).
#' @param edgeWidth Width to be used for all edges before scaling
#' if \code{numCol} is \code{NULL}; otherwise ignored.
#' @param edgeColor Color used for edges.
#' @param edgeScales Edge width scales. Must be a numeric vector of size
#' 2 (minimum and maximum).
#'
#' @return An object of class \code{ggraph}.
#'
#' @examples
#' df <- data.frame(gene1 = paste0('G', c(1, 2, 5, 6, 7, 17)),
#' gene2 = paste0('G', c(2, 5, 8, 11, 11, 11)),
#' rank = c(1, 1, 3, 3, 3, 3))
#' networkPlot(df, numCol='rank', numColType='ranks')
#'
#' @export
#'
networkPlot <- function(df,
                        title = NULL,
                        numCol = NULL,
                        numColType = c('weights', 'ranks'),
                        nodeSize = 10,
                        nodeTextSize = 2.3,
                        palette = 'grDevices::Spectral',
                        nodeColor = NULL,
                        edgeWidth = 1,
                        edgeColor = 'black',
                        edgeScales = c(0.2, 0.6),
                        ...){


    if (length(edgeScales) != 2)
        stop('`edgeScales` must be a numeric vector of length 2.')

    if(!is.null(numCol)){
        if (!numCol %in% colnames(df))
            stop('The edge width column was not found. ',
                 'If there is no edge width column, set `numCol` to NULL.')

        numColType <- match.arg(numColType, c('weights', 'ranks'))
        if (numColType == 'ranks')
            df <- networkPlotDF(df, numCol) else
                colnames(df)[which(colnames(df) %in% numCol)] <- 'weight'
    }

    tblGraph <- as_tbl_graph(df, directed=FALSE)
    df <- connectedComponents(df)
    vertexComp <- vertexComponents(df)
    nComp <- length(unique(vertexComp))
    tblGraph <- mutate(activate(tblGraph, 'vertices'),'component'=vertexComp)

    p <- ggraph(tblGraph, layout='nicely') +
        geom_edge_link(aes(width=if (is.null(numCol))
                                     edgeWidth else .data[['weight']]),
                       color=edgeColor) +
        scale_edge_width(range=edgeScales) +
        theme_void() +
        theme(legend.position='none') +
        scale_color_manual(values=paletteer_c(palette, nComp))

    if (is.null(nodeColor))
        p <- p + geom_node_point(aes(color=.data[['component']]),
                                 size=nodeSize) else
                                 p <- p + geom_node_point(color=nodeColor,
                                                          size=nodeSize)

    p <- p + geom_node_text(aes(label=.data[['name']]), size=nodeTextSize)
    p <- centerTitle(p, title, ...)
    return(p)
}

#' @importFrom rlang .data
#'
NULL

#' Plot bars for item counts grouped and colored by class
#'
#' This function plots bars for item counts grouped and colored by class.
#'
#' @inheritParams documentFun
#' @param df A data frame with at least three columns. Its first column
#' (categorical) colors the plot bars. The second column (categorical)
#' labels the plots bars. The third column (numeric) sets the bar lengths.
#' @param decreasing Whether to display the bars in decreasing order of length.
#' @param valueCutoff Cutoff used for filtering the input data frame based on
#' the third (value) column. Only values above this cutoff will be displayed on
#' the plot.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#'  df <- data.frame(Class = sample(paste0('C', seq(13)), 25, replace=TRUE),
#'  Item = paste0('I', seq(25)),
#'  Value = runif(25, 0.5, 1))
#'  classPlot(df)
#'
#'  df <- data.frame(Class = sample(paste0('C', seq(13)), 25, replace=TRUE),
#'  Item = sample(paste0('I', seq(21)), 25, replace=TRUE),
#'  Value = runif(25, 0.5, 1))
#'  classPlot(df)
#'
#' @export
#'
classPlot <- function(df,
                      title = NULL,
                      xLab = 'Value',
                      yLab = 'Item',
                      legendTitle = 'Class',
                      palette = 'Spectral',
                      labelSize = 2.5,
                      labelColor ='black',
                      legendTitleSize = 10,
                      legendTextSize = 10,
                      axisTextSize = 12,
                      axisTitleSize = 12,
                      decreasing = TRUE,
                      valueCutoff = 0,
                      ...){
    df <- df[df[, 3] > valueCutoff, ]
    nClasses <- length(unique(df[, 1]))
    df <- df[order(df[, 3], decreasing=decreasing), ]
    df[, 1] <- factor(df[, 1], levels=unique(df[, 1]))
    df[, 4] <- make.unique(as.character(df[, 2]))
    df[, 4] <- factor(df[, 4], levels=rev(df[, 4]))
    p <- ggplot(data=df,
                aes(fill=.data[[names(df)[1]]],
                    x=.data[[names(df)[3]]],
                    y=.data[[names(df)[4]]])) +
        geom_bar(position='stack', stat='identity') +
        theme_classic() +
        theme(legend.text=element_text(size=legendTextSize),
              legend.title=element_text(size=legendTitleSize),
              axis.ticks.y=element_blank(),
              axis.text.y=element_blank(),
              axis.text.x=element_text(size=axisTextSize),
              axis.title.x=element_text(size=axisTitleSize)) +
        labs(x=xLab, y=yLab, fill=legendTitle) +
        scale_fill_manual(values=hcl.colors(nClasses, palette)) +
        geom_text(aes(x=.data[[names(df)[3]]] / 2,
                      y=.data[[names(df)[4]]],
                      label=.data[[names(df)[2]]]),
                  size=labelSize,
                  color=labelColor)
    p <- centerTitle(p, title, ...)
    return(p)
}

#' Find the connected components of a graph represented as a data frame
#'
#' This function finds the connected components of a graph represented as
#' a data frame.
#'
#' @param df A data frame with two categorical columns representing the edges
#' of a graph.
#' @param colName Name of the connected components column to be added.
#' @return A data frame with a column indicating the connected component of
#' each edge.
#'
#' @examples
#' df <- data.frame(
#' gene1 = paste0('G', c(1, 2, 6, 7, 8, 9,
#' 11, 25, 32, 17, 18)),
#' gene2 = paste0('G', c(2, 8, 8, 8, 1, 25,
#' 32, 24, 24, 26, 26))
#' )
#' connectedComponents(df)
#'
#' @export
#'
connectedComponents <- function(df, colName = 'component'){
    if(!nrow(df))
        stop('The dataframe has no rows.')
    df[[colName]] <- -1
    rownames(df) <- seq(dim(df)[1])
    vertices <- unique(c(df[, 1], df[, 2]))
    seen <- c()
    nextComp <- 1
    for (v in vertices){
        if (v %in% seen)
            next
        currVertices <- c(v)
        while (length(currVertices)){
            v <- currVertices[1]
            leftdf <- subset(df, df[, 1] == v)
            rightdf <- subset(df, df[, 2] == v)
            seen <- c(seen, v)
            newEdges <- as.integer(c(rownames(leftdf), rownames(rightdf)))
            df[newEdges, colName] <- nextComp
            neighbors <- setdiff(c(leftdf[, 2], rightdf[, 1]),
                                 c(currVertices, seen))
            currVertices <- c(currVertices, neighbors)
            currVertices <- currVertices[-1]
        }
        nextComp <- nextComp + 1
    }
    df[, colName] <- factor(df[, colName])
    return(df)
}

#' Return the connected components of vertices
#'
#' This function returns the connected components of vertices from a graph
#' data frame in which edges have been assigned connected components.
#'
#' @param df A data frame with two categorical columns representing graph edges
#' and a connected components column.
#' @param colName Name of the connected components column.
#'
#' @return A factor vector representing the connected component of each
#' vertex.
#'
#' @examples
#' df <- data.frame(gene1 = c('A', 'B', 'C', 'A'),
#' gene2 = c('B', 'D', 'F', 'G'),
#' component = c(1, 1, 2, 1))
#' vertexComponents(df)
#'
#' @export
#'
vertexComponents <- function(df, colName = 'component'){
    vertices <- unique(c(df[, 1], df[, 2]))
    boxes <- lapply(unique(df[, colName]), function(x) {
        compDF <- df[df[, colName] == x, ]
        return(unique(c(compDF[, 1], compDF[, 2])))
    })
    nBoxes <- length(boxes)
    allocations <- unlist(lapply(seq(nBoxes), function(i)
        setNames(rep(i, length(boxes[[i]])),boxes[[i]])))
    vertexComp <- as.factor(allocations[vertices])
    return(vertexComp)
}

#' Construct the convex hull of a set of points
#'
#' This function constructs the convex hull of a set of points.
#'
#' @details The points must be provided as a data frame with two columns.
#'
#' @param pointsDF A data frame with the x and y coordinates of the points.
#' @param hullIndices Precalculated hull indices. Default is \code{NULL}: hull
#' indices are not provided, but they are calculated by \code{convexHull}.
#'
#' @return A data frame with two columns representing the points on the convex
#' hull.
#'
#' @examples
#' pointsDF <- data.frame(a = c(1, 2, 2, 3, 3, 4, 5, 6, 8, 6,
#' 7, 8, 6, 8, 10, 3, 1),
#' b = c(2, 3, 4, 8, 5, 6, 5, 4, 8, 11, 13, 14, 2, 1, 2, 14, 9))
#' hull <- convexHull(pointsDF)
#'
#' @export
#'
convexHull <- function(pointsDF, hullIndices=NULL){
    if(!is.null(hullIndices))
        hull <- pointsDF[hullIndices, c(1, 2)] else
            hull <- pointsDF[chull(pointsDF[, 1], pointsDF[, 2]), c(1, 2)]
    colnames(hull) <- c('x', 'y')
    return(hull)
}

#' Construct a data frame of segments from a data frame of points
#'
#' This function constructs a data frame of segments from a data frame of
#' points.
#'
#' @param pointsDF A data frame with the x and y coordinates of the points.
#' Each point must appear only once.
#' @param joinEnds Whether to join the last point with the first one.
#'
#' @return A data frame of segments represented using four columns
#' (\code{x}, \code{y}, \code{xEnd}, \code{yEnd}).
#'
#' @examples
#' pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
#' 12, 13, 15, 16),
#' y = c(1, 1, 2, 3, 3, 2, 1, 2, 1))
#'
#' hullIndices <- grDevices::chull(pointsDF[, 1], pointsDF[, 2])
#' hull <- convexHull(pointsDF, hullIndices)
#' pointsToSegments(hull)
#'
#' @export
#'
pointsToSegments <- function(pointsDF,
                             joinEnds = TRUE){
    df <- data.frame(x = pointsDF[seq_len(nrow(pointsDF) - 1), 1],
                     y = pointsDF[seq_len(nrow(pointsDF) - 1), 2],
                     xEnd = pointsDF[seq(2, nrow(pointsDF)), 1],
                     yEnd = pointsDF[seq(2, nrow(pointsDF)), 2])
    if(joinEnds)
        df <- rbind(df, c(df$xEnd[nrow(df)],
                          df$yEnd[nrow(df)],
                          df$x[1],
                          df$y[1]))
    return(df)
}

#' Check if a point is on a segment
#'
#' This function checks if a point P is on a segment AB.
#'
#' @param xPoint x coordinate of point P.
#' @param yPoint y coordinate of point P.
#' @param xStart x coordinate of point A.
#' @param yStart y coordinate of point A.
#' @param xEnd x coordinate of point B.
#' @param yEnd y coordinate of point B.
#'
#' @return Logical; whether the point is on the segment.
#'
#' @examples
#' isPointOnSeg(2, 3, 1, 2, 3, 4)
#' isPointOnSeg(2, 3, 1, 2, 3, 8)
#' isPointOnSeg(4, 5, 1, 2, 3, 4)
#'
#' @export
#'
isPointOnSeg <- function(xPoint, yPoint, xStart, yStart, xEnd, yEnd){
    if (xPoint < min(xStart, xEnd) | xPoint > max(xStart, xEnd) |
        yPoint < min(yStart, yEnd) | yPoint < min(yStart, yEnd))
        return(FALSE)
    slope <- (yEnd - yStart) / (xEnd - xStart)
    yIntercept <- yStart - slope * xStart
    return (is(all.equal(slope * xPoint + yIntercept, yPoint))[1] == 'logical')
}

#' Check if a point is on a polygon boundary
#'
#' This function checks if a point P is on a polygon boundary.
#'
#' @inheritParams isPointOnSeg
#' @param boundary A data frame with four columns representing segments
#' comprising the boundary.
#'
#' @return Logical; whether the point is on the boundary.
#'
#' @examples
#' pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
#' 12, 13, 15, 16),
#' y = c(1, 1, 2, 3, 3, 2,
#' 1, 2, 1))
#'
#' hullIndices <- grDevices::chull(pointsDF[, 1], pointsDF[, 2])
#' hull <- convexHull(pointsDF, hullIndices)
#' hullSegments <- pointsToSegments(hull)
#'
#' isPointOnBoundary(2, 3, hullSegments)
#'
#' @export
#'
isPointOnBoundary <- function(xPoint, yPoint, boundary)
    return(as.logical(max(apply(boundary, 1, function(x){
        v <- unlist(x)
        names(v) <- c()
        return(isPointOnSeg(xPoint, yPoint, v[1], v[2], v[3], v[4]))
    }))))

test_that("classPlot returns a ggplot object", {
    df <- data.frame(Class = sample(paste0('C', seq(13)), 25, replace=TRUE),
                     Item = paste0('I', seq(25)),
                     Value = runif(25, 0.5, 1))
    p <- classPlot(df)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)

    df <- data.frame(Class = sample(paste0('C', seq(13)), 25, replace=TRUE),
                     Item = sample(paste0('I', seq(21)), 25, replace=TRUE),
                     Value = runif(25, 0.5, 1))
    p <- classPlot(df)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("correlationPlot returns a ggplot object", {
    mat <- matrix(runif(100, -1, 1), nrow=10)
    colnames(mat) <- paste0('I', seq(10))
    mat <- round(cor(mat), 2)
    p <- correlationPlot(mat)
    class(p)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that('a', {
    x <- 2
    expect_equal(x, 2)
})

test_that("densityPlot returns a ggplot object", {
    x <- c(1, 1, 2, 7, 8, 0, 16, 1, 1.2, 32, 7, 5, 1.1, 1.2, 1.2)
    y <- c(1, 1.1, 0.5, 8, 18, 4, 6, 0.9, 1, 6, -24, -28, 1, 0.8, 0.9)
    z <- round(runif(15, 75, 100), 2)
    df <- data.frame(x, y, z)
    rownames(df) <- paste0('p', rownames(df))
    p <- densityPlot(df)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("hullPlot works", {
    pointsDF <- data.frame(x = c(1, 2, 4, 7, 10, 12, 13, 15, 16),
                           y = c(1, 1, 2, 3, 3, 2,1, 2, 1))

    p <- hullPlot(pointsDF, xInt=9, yInt=2.5, lineColor='maroon', lineWidth=1,
             lineType='solid')
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)


    p <- hullPlot(pointsDF, 'Hull plot', 7, 1.5)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)

    expect_error(hullPlot(pointsDF, 'Hull plot', 1, 2))
    expect_error(hullPlot(pointsDF, 'Hull plot', 4, 2))

    p <- hullPlot(pointsDF, 'Hull plot', 4.1, 2)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)

    expect_error(hullPlot(pointsDF, 'Hull plot', 15.5, 1.5))

    p <- hullPlot(pointsDF, 'Hull plot',15.4, 1.5)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)

    rownames(pointsDF) <- paste0('P', seq(9))
    p <- hullPlot(pointsDF, 7, 1.5, labeledPoints=c('P1', 'P4', 'P9'))
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("networkPlot returns a ggraph object", {
    df <- data.frame(gene1 = paste0('G', c(1, 2, 5, 6, 7, 17)),
                     gene2 = paste0('G', c(2, 5, 8, 11, 11, 11)),
                     rank = c(1, 1, 3, 3, 3, 3),
                     weight = c(8, 2, 1, 12, 3, 4))
    p <- networkPlot(df, numCol='rank', numColType='ranks')
    expect_equal(is(p), 'ggraph')
    p <- networkPlot(df, numCol='weight', numColType='weights')
    expect_equal(is(p), 'ggraph')
    p <- networkPlot(df)
    expect_equal(is(p), 'ggraph')
    p <- networkPlot(df, nodeColor='orange', edgeColor='green4')
    expect_equal(is(p), 'ggraph')
})

test_that("radialPlot returns a gg object", {
    valuesDF <- data.frame(Protein = paste0('P', seq(20)),
                            Value = sample(10, 20, replace=TRUE),
                            Group = sample(3, 20, replace=TRUE))
    p <- radialPlot(valuesDF, seed=200, breakDensity=8)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("rankPlot returns a gg object", {
    df <- do.call(cbind, lapply(seq(30), function(i) sample(10, 10)))
    rownames(df) <- paste0('M', seq(10))
    colnames(df) <- paste0('R', seq(30))
    p <- rankPlot(df)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
    p <- rankPlot(df, sigDigits=2, labelScalingFactor=0.85, labelOffset=0.07)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("riverPlot returns a gg object", {
    df <- data.frame(x = sample(c('a','b', 'c', 'd', 'e', 'f'), 20,
                                replace=TRUE),
                     y = sample(c('p','q', 'r', 's', 't', 'u', 'v', 'w'), 20,
                                replace=TRUE),
                     z = runif(20, 1, 3))
    p <- riverPlot(df)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("reorderDF works", {
    df <- data.frame(a = c(2, 4, 1, 3), b = c(2, 8, 3, 19))
    df <- reorderDF(df)
    expect_identical(df[, 1], c(1, 2, 3, 4))
    expect_identical(df[, 2], factor(c(3, 2, 19, 8), levels=c(3, 2, 19, 8)))
})

test_that("tilePlot returns a gg object", {
    mat <- matrix(round(runif(100, 0, 1), 2), nrow=10)
    rownames(mat) <- paste0('R', seq(10))
    colnames(mat) <- paste0('C', seq(10))
    p <- tilePlot(mat, limits=c(0, 1), showNumbers=FALSE)
    expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
})

test_that("connectedComponents works", {
    df <- data.frame(
        gene1 = paste0('G', c(1, 2, 6, 7, 8, 9,
                              11, 25, 32, 17, 18)),
        gene2 = paste0('G', c(2, 8, 8, 8, 1, 25,
                              32, 24, 24, 26, 26)))

    comps <- as.numeric(connectedComponents(df)$component)
    expect_identical(comps, c(rep(1, 5), rep(2, 4), 3, 3))
})

test_that("vertexComponents works", {
    df <- data.frame(gene1 = c('A', 'B', 'C', 'A'),
                     gene2 = c('B', 'D', 'F', 'G'),
                     component = c(1, 1, 2, 1))
    expect_identical(vertexComponents(df),
                     as.factor(setNames(c(1, 1, 2, 1, 2, 1),
                                        c('A', 'B', 'C', 'D', 'F', 'G'))))
})

test_that("convexHull works", {
    pointsDF <- data.frame(a = c(1, 2, 2, 3, 3, 4, 5, 6, 8, 6, 7, 8, 6, 8, 10, 3, 1),
                           b = c(2, 3, 4, 8, 5, 6, 5, 4, 8, 11, 13, 14, 2, 1, 2, 14, 9))
    hull <- convexHull(pointsDF)
    rownames(hull) <- NULL
    expectedHull <- data.frame(x = c(10, 8, 1, 1, 3, 8),
                               y = c(2, 1, 2, 9, 14, 14))
    expect_identical(hull, expectedHull)
})

test_that("pointsToSegments works", {
    pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
                                 12, 13, 15, 16),
                           y = c(1, 1, 2, 3, 3, 2, 1, 2, 1))

    hullIndices <- grDevices::chull(pointsDF[, 1], pointsDF[, 2])
    hull <- convexHull(pointsDF, hullIndices)
    hullSegments <- pointsToSegments(hull)
    expectedHullSegments <- data.frame(x = c(16, 1, 7, 10, 15),
                                       y = c(1, 1, 3, 3, 2),
                                       xEnd = c(1, 7, 10, 15, 16),
                                       yEnd = c(1, 3, 3, 2, 1))
    expect_identical(hullSegments, expectedHullSegments)
})

test_that("isPointOnSeg works", {
    expect_true(isPointOnSeg(2, 3, 1, 2, 3, 4))
    expect_false(isPointOnSeg(2, 3, 1, 2, 3, 8))
    expect_false(isPointOnSeg(4, 5, 1, 2, 3, 4))
})

test_that("isPointOnBoundary works", {
    pointsDF <- data.frame(x = c(1, 2, 4, 7, 10, 12, 13, 15, 16),
                           y = c(1, 1, 2, 3, 3, 2, 1, 2, 1))

    hullIndices <- grDevices::chull(pointsDF[, 1], pointsDF[, 2])
    hull <- convexHull(pointsDF, hullIndices)
    hullSegments <- pointsToSegments(hull)
    expect_false(isPointOnBoundary(2, 3, hullSegments))
    expect_true(isPointOnBoundary(12, 2.6, hullSegments))
})

test_that("volcanoPlot returns a gg object", {
    filePath <- system.file('extdata', 'volcanoPlot.qs2', package='henna')
    df <- qs2::qs_read(filePath)
    if (requireNamespace("EnhancedVolcano", quietly=TRUE)){
        p <- suppressWarnings(volcanoPlot(df,
                         title='Volcano plot - beta cells',
                         pvalThr=1e-10,
                         logFCThr=1,
                         labPvalThr=1e-150,
                         labLogFCThr=5.3))
        expect_equal(length(intersect(is(p), c('gg', 'ggplot2::ggplot'))), 1)
    } else expect_error(volcanoPlot(df))
})

#'@importFrom abdiv euclidean
#'@importFrom ggplot2 aes after_stat coord_fixed element_blank element_text expand_limits geom_bar geom_label geom_point geom_polygon geom_segment geom_text geom_tile ggplot ggproto ggtitle labs margin scale_color_discrete scale_color_manual scale_fill_gradientn scale_fill_manual scale_fill_viridis_d stat_density_2d theme theme_bw theme_classic theme_linedraw theme_minimal theme_void xlab
#'@importFrom stats density runif setNames
#'
NULL

#' Add a centered title to a plot
#'
#' This function adds a centered title to a ggplot object.
#'
#' @param p A ggplot object.
#' @param title Plot title.
#' @param ... Other arguments passed to \code{ggplot2::element_text}.
#'
#' @return A ggplot object.
#'
#' @export
#'
centerTitle <- function(p, title, ...)
    return(p + ggtitle(title) + theme(plot.title=element_text(hjust=0.5, ...)))

#' Add a nearest neighbor column to a data frame of points
#'
#' This function adds a nearest neighbor column to a data frame of points.
#'
#' @inheritParams dfPoint
#'
#' @return A data frame with an added nearest neighbor column.
#'
#' @noRd
#'
addNNCol <- function(df, pointCoords = c(1, 2)){
    df$nn <- vapply(rownames(df), function(i)
        names(which.min(vapply(rownames(df), function(j)
            dfEuclidean(df, i, j, pointCoords, replaceZero=Inf),
            numeric(1)))), character(1))
    return(df)
}

#' Extract point from data frame of points.
#'
#' This function extracts a point from a data frame of points.
#'
#' @param df A data frame containing the coordinates
#' of the points.
#' @param i Row name or index.
#' @param pointCoords The row names or indices of point coordinates.
#'
#' @return A numeric vector containing the point coordinates.
#'
#' @keywords internal
#'
dfPoint <- function(df, i, pointCoords = c(1, 2))
    return(unlist(df[i, pointCoords]))

#' Find the Euclidean distance between two points in a data frame
#'
#' This function finds the Euclidean distance between two points in a data
#' frame.
#'
#' @inheritParams dfPoint
#' @param j Row name or index
#' @param replaceZero Value to replace zero with. If set to 0, no replacement
#' will occur.
#'
#' @return A numeric value.
#'
#' @keywords internal
#'
dfEuclidean <- function(df, i, j, pointCoords = c(1, 2), replaceZero = 0){
    eDist <- euclidean(dfPoint(df, i, pointCoords),
                       dfPoint(df, j, pointCoords))
    if (!eDist)
        return(replaceZero)
    return(eDist)
}

#' Expand range of a vector
#'
#' This function expands the range of a vector by adding a new minimum and
#' maximum.
#'
#' @param v A numeric vector.
#' @param expandPerc Percentage by which to expand the vector.
#'
#' @return A numeric vector of size 2, containing the new minimum and maximum.
#'
#' @noRd
#'
expandRange <- function(v, expandPerc = 10){
    vMin <- min(v)
    vMax <- max(v)
    vRange <- vMax - vMin
    expVal <- vRange * expandPerc / 100
    return(c(vMin - expVal, vMax + expVal))
}

#' Find the coordinates where a vertical or horizontal line intersects the hull
#'
#' This function finds the coordinates where a vertical or horizontal line
#' intersects the hull.
#'
#'
#' @param df A four-column data frame representing segments.
#' @param axis An integer representing the axis intersected by the vertical or
#' horizontal line, x (1) or y (2).
#' @param axisIntersect The coordinate where the vertical or horizontal line
#' intersects the relevant axis.
#'
#' @return A vector of size two representing the coordinates of the two
#' intersection points between the vertical or horizontal line and the convex
#' hull on the axis different from the input axis.
#'
#' @keywords internal
#'
borderCoords <- function(df, axis, axisIntersect){

    if(is.null(axisIntersect))
        return(c('None', 'None'))

    otherAxis <- axis %% 2 + 1
    axisEnd <- axis + 2
    otherAxisEnd <- otherAxis + 2
    axisVals <- df[, axis]
    axisEndVals <- df[, axisEnd]

    coords <- c(df[axisVals == axisIntersect, ][, otherAxis])

    if (length(coords) < 2){
        df <- df[axisVals < axisIntersect & axisEndVals > axisIntersect |
                     axisVals > axisIntersect & axisEndVals < axisIntersect, ]

        axisVals <- df[, axis]
        axisEndVals <- df[, axisEnd]
        otherAxisVals <- df[, otherAxis]
        otherAxisEndVals <- df[, otherAxisEnd]

        df$diffRatio <- (otherAxisEndVals - otherAxisVals) /
            (axisEndVals  - axisVals)
        df$newCoord <- df$diffRatio * (axisIntersect - axisVals) +
            otherAxisVals

        coords <- c(coords, df$newCoord)
    }

    return(sort(coords))
}

#' Split the convex hull in two along an input line
#'
#' This function splits the convex hull in two along an input vertical or
#' horizontal line.
#'
#'
#' @inheritParams documentFun
#' @param p A ggplot object representing the hull.
#' @inheritParams convexHull
#' @inheritParams borderCoords
#' @param pointCoords The coordinates of the input points on the axis
#' perpendicular to the input border line.
#' @param borderPoints The points where the border line intersects the
#' convex hull.
#' @param alpha Opaqueness level.
#'
#' @return A ggplot object showing the hull split in two parts along the input
#' line.
#'
#' @keywords internal
#'
splitInTwo <- function(p,
                       pointsDF,
                       axisIntersect,
                       pointCoords,
                       borderPoints,
                       legendLabs = paste0('Group ', seq(2)),
                       alpha = 0.5){
    df1 <- rbind(pointsDF[pointCoords < axisIntersect, ], borderPoints)
    df2 <- rbind(pointsDF[pointCoords > axisIntersect, ], borderPoints)

    hullSegments1 <- pointsToSegments(convexHull(df1))
    hullSegments2 <- pointsToSegments(convexHull(df2))

    p <- p + geom_polygon(data=hullSegments1, aes(x=.data[['x']],
                                                  y=.data[['y']],
                                                  fill=legendLabs[1]),
                          alpha=alpha)
    p <- p + geom_polygon(data=hullSegments2, aes(x=.data[['x']],
                                                  y=.data[['y']],
                                                  fill=legendLabs[2]),
                          alpha=alpha)
    return(p)
}

#' Find the coordinates of the points establishing the four divisions of
#' the hull
#'
#' This function finds the coordinates of the points establishing the
#' four divisions of the hull
#'
#' @inheritParams convexHull
#' @param xInt The coordinate where the vertical line intersects the x axis.
#' @param yInt The coordinate where the horizontal line intersects the y axis.
#' @param vCoords The y coordinates of the two points where the vertical line
#' intersects the convex hull.
#' @param hCoords The x coordinates of the two points where the horizontal line
#' intersects the convex hull.
#'
#' @return A data frame with 2 columns representing the 12 points (not unique)
#' determining the boundaries of the hull divisions.
#'
#' @keywords internal
#'
quadBorders <- function(pointsDF, xInt, yInt, vCoords, hCoords){
    a <- c(xInt, hCoords[1], xInt,
           xInt, hCoords[2], xInt,
           xInt, hCoords[1], xInt,
           xInt, hCoords[2], xInt)
    b <- c(yInt, yInt, vCoords[1],
           yInt, yInt, vCoords[2],
           yInt, yInt, vCoords[2],
           yInt, yInt, vCoords[1])
    df <- data.frame(a, b)
    colnames(df) <- colnames(pointsDF)
    return(df)
}

#' Split the convex hull in four parts along two input lines
#'
#' This function splits the convex hull in two parts along a vertical and a
#' horizontal line.
#'
#'
#' @param p A ggplot object representing the hull.
#' @inheritParams splitInTwo
#' @inheritParams quadBorders
#' @param borderPoints The points where the border lines intersects the
#' convex hull.
#'
#' @return A ggplot object showing the hull split in four parts along the input
#' axes.
#'
#' @noRd
#'
splitInFour <- function(p,
                        pointsDF,
                        xInt,
                        yInt,
                        borderPoints,
                        legendLabs = paste0('Group ', seq(4)),
                        alpha = 0.5){
    df1 <- rbind(pointsDF[pointsDF[, 1] < xInt & pointsDF[, 2] < yInt, ],
                 borderPoints[c(1, 2, 3), ])
    df2 <- rbind(pointsDF[pointsDF[, 1] > xInt & pointsDF[, 2] > yInt, ],
                 borderPoints[c(4, 5, 6), ])
    df3 <- rbind(pointsDF[pointsDF[, 1] < xInt & pointsDF[, 2] > yInt, ],
                 borderPoints[c(7, 8, 9), ])
    df4 <- rbind(pointsDF[pointsDF[, 1] > xInt & pointsDF[, 2] < yInt, ],
                 borderPoints[c(10, 11, 12), ])

    hullSegments1 <- pointsToSegments(convexHull(df1))
    hullSegments2 <- pointsToSegments(convexHull(df2))
    hullSegments3 <- pointsToSegments(convexHull(df3))
    hullSegments4 <- pointsToSegments(convexHull(df4))

    p <- p + geom_polygon(data=hullSegments1, aes(x=.data[['x']],
                                                  y=.data[['y']],
                                                  fill=legendLabs[1]),
                          alpha=alpha)
    p <- p + geom_polygon(data=hullSegments2, aes(x=.data[['x']],
                                                  y=.data[['y']],
                                                  fill=legendLabs[2]),
                          alpha=alpha)
    p <- p + geom_polygon(data=hullSegments3, aes(x=.data[['x']],
                                                  y=.data[['y']],
                                                  fill=legendLabs[3]),
                          alpha=alpha)
    p <- p + geom_polygon(data=hullSegments4, aes(x=.data[['x']],
                                                  y=.data[['y']],
                                                  fill=legendLabs[4]),
                          alpha=alpha)
    return(p)
}


#' Split the convex hull in four parts along two input lines
#'
#' This function splits the convex hull in two parts along a vertical and a
#' horizontal line.
#'
#'
#' @inheritParams splitInTwo
#' @param hullSegments Data frame of segments that define the convex hull.
#' @inheritParams quadBorders
#' @param lineColor The color of the horizontal and vertical dividing lines,
#' if provided. If \code{NULL}, no dividing lines will be drawn, though the
#' hull will still be split along these lines if \code{xInt}
#' and/or \code{yInt}are not \code{NULL}.
#' @param lineWidth The width of the horizontal and vertical dividing lines.
#' Ignored if \code{lineColor} is \code{NULL}.
#' @param lineType The type of the horizontal and vertical dividing lines.
#' Choose between 'dashed','solid', 'dotted', 'dotdash', 'longdash' and
#' 'twodash'. Default is 'dashed'. Ignored if \code{lineColor} is \code{NULL}.
#'
#' @return An object of class \code{gg} showing the hull split along the input
#' axes.
#'
#' @keywords internal
#'
splitHull <- function(p,
                      pointsDF,
                      hullSegments,
                      xInt = NULL,
                      yInt = NULL,
                      lineColor = 'navy',
                      lineWidth = 0.3,
                      lineType = c('dashed','solid', 'dotted',
                                   'dotdash', 'longdash', 'twodash'),
                      legendLabs = paste0('Group ', seq(4)),
                      alpha = 0.2){

    lineType <- match.arg(lineType, c('dashed','solid', 'dotted',
                                      'dotdash', 'longdash', 'twodash'))

    vCoords <- borderCoords(hullSegments, 1, xInt)
    hCoords <- borderCoords(hullSegments, 2, yInt)

    if(!is.null(lineColor)){
        if (!is.null(xInt))
            p <- p + geom_segment(aes(x=xInt,
                                      y=vCoords[1],
                                      xend=xInt,
                                      yend=vCoords[2]),
                                  color=lineColor,
                                  linewidth=lineWidth,
                                  linetype=lineType)
        if (!is.null(yInt))
            p <- p + geom_segment(aes(x=hCoords[1],
                                      y=yInt,
                                      xend=hCoords[2],
                                      yend=yInt),
                                  color=lineColor,
                                  linewidth=lineWidth,
                                  linetype=lineType)
    }

    if(is.null(xInt) & is.null(yInt))
        return(p + geom_polygon(data=hullSegments,
                                aes(x=.data[['x']],
                                    y=.data[['y']],
                                    fill=legendLabs[1]),
                                alpha=alpha))
    if(is.null(xInt))
        return(splitInTwo(p, pointsDF, yInt, pointsDF[, 2],
                          list(hCoords, c(yInt, yInt)),
                          legendLabs))

    if(is.null(yInt))
        return(splitInTwo(p, pointsDF, xInt, pointsDF[, 1],
                          list(c(xInt, xInt), vCoords),
                          legendLabs, alpha))


    return(splitInFour(p, pointsDF, xInt, yInt,
                       quadBorders(pointsDF, xInt, yInt, vCoords, hCoords),
                       legendLabs, alpha))
}

#' Plot the convex hull of a set of points
#'
#' This function plots the convex hull of a set of points. It can also draw
#' a vertical or a horizontal line (or both), dividing the hull into areas of
#' different colors.
#'
#' @inheritParams documentFun
#' @inheritParams convexHull
#' @inheritParams splitHull
#' @inheritParams createLabelDFTemplate
#' @param hullWidth Width of the convex hull. If 0 (as default), the convex
#' hull will not be displayed.
#' @param labelSize Label size. Ignored if \code{labelDF} is \code{NULL}.
#' @param labelColor Label color. Ignored if \code{labelDF} is \code{NULL}.
#' @param maxOverlaps Maximum overlaps. Ignored if \code{labelDF}
#' is \code{NULL}.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
#' 12, 13, 15, 16),
#' y = c(1, 1, 2, 3, 3, 2,
#' 1, 2, 1))
#' hullPlot(pointsDF, 'Hull plot', 7, 1.5)
#'
#' @export
#'
hullPlot <- function(pointsDF,
                     title = NULL,
                     xInt = NULL,
                     yInt = NULL,
                     palette = hpColors(),
                     lineColor = 'navy',
                     lineWidth = 0.3,
                     lineType = c('dashed','solid', 'dotted',
                                  'dotdash', 'longdash', 'twodash'),
                     hullWidth = 0,
                     xLab = NULL,
                     yLab = NULL,
                     legendLabs = paste0('Group ', seq(4)),
                     pointSize = 1,
                     pointShape = 4,
                     alpha = 0.2,
                     labeledPoints = NULL,
                     labelOutside = FALSE,
                     labXThr = NULL,
                     labYThr = NULL,
                     labelType = c('free', 'boxed'),
                     labelSize = 2.5,
                     labelColor = 'black',
                     labelRepulsion = 1,
                     labelPull = 0,
                     maxOverlaps = 10,
                     legendPos = 'bottom',
                     legendTextSize = 10,
                     axisTextSize = 12,
                     axisTitleSize = 12,
                     ...){

    labelType <- match.arg(labelType, c('free', 'boxed'))

    if (nrow(pointsDF) < 2)
        stop('The hull plot requires at least two points.')

    pointsX <- pointsDF[, 1]
    pointsY <- pointsDF[, 2]

    if (!is.null(xInt))
        if (xInt <= min(pointsX) | xInt >= max(pointsX))
            stop('xInt is outside the (min(X), max(X)) interval.')

    if (!is.null(yInt))
        if (yInt <= min(pointsY) | yInt >= max(pointsY))
            stop('yInt is outside the (min(Y), max(Y)) interval.')

    if(nrow(pointsDF) > 2)
        pointsDF <- pointsDF[, c(1, 2)]

    hullIndices <- chull(pointsX, pointsY)
    hull <- convexHull(pointsDF, hullIndices)
    hullSegments <- pointsToSegments(hull)

    if (!is.null(xInt) & !is.null(yInt)){
        if(isPointOnBoundary(xInt, yInt, hullSegments))
            stop('The (xInt, yInt) point is on the boundary of the polygon',
                 ' determined by the input points. It must be inside.')

        tempPointsDF <- rbind(pointsDF, c(xInt, yInt))
        tempHullIndices <- chull(tempPointsDF)
        if (!identical(sort(hullIndices),
                       sort(tempHullIndices)))
            stop('The (xInt, yInt) point is outside the polygon',
                 ' determined by the input points. It must be inside.')

        slope <- (hullSegments$yEnd - hullSegments$y) /
            (hullSegments$xEnd - hullSegments$x)
        minDiff <- min(abs((yInt - hullSegments$y) /
                    (xInt - hullSegments$x) - slope))
        if(!minDiff)
            stop('The (xInt, yInt) point is on the boundary of the polygon',
                 ' determined by the input points. It must be inside.')
    }

    p <- ggplot() + theme_classic() +
        labs(x=xLab, y=yLab) +
        theme(legend.position=legendPos,
              legend.text=element_text(size=legendTextSize),
              legend.title=element_blank(),
              axis.text=element_text(size=axisTextSize),
              axis.title=element_text(size=axisTitleSize))

    if(hullWidth)
        p <- p + geom_segment(data=hullSegments,
                              aes(x=.data[['x']],
                                  y=.data[['y']],
                                  xend=.data[['xEnd']],
                                  yend=.data[['yEnd']]),
                              linewidth=hullWidth)

    p <- splitHull(p, pointsDF, hullSegments, xInt, yInt,
                   lineColor, lineWidth, lineType,
                   legendLabs, alpha)
    p <- p + scale_fill_manual(values=palette, labels=legendLabs)

    p <- p + geom_point(data=pointsDF, aes(x=.data[[names(pointsDF)[1]]],
                                           y=.data[[names(pointsDF)[2]]]),
                   size=pointSize, shape=pointShape)

    labelDF <- createLabelDFHull(pointsDF, 1, 2, labeledPoints,
                                 labelOutside, labXThr, labYThr)

    if(!is.null(labelDF))
        p <- labelPoints(p, labelDF, rownames(labelDF), labelType, labelSize,
                         labelColor, labelRepulsion, labelPull, maxOverlaps)

    p <- centerTitle(p, title, ...)
    return(p)
}

#' @importFrom dplyr count
#' @importFrom ggeasy easy_remove_axes
#' @importFrom ggforce geom_circle
#' @importFrom ggnewscale new_scale_color new_scale_fill
#' @importFrom ggrepel geom_label_repel geom_text_repel
#' @importFrom viridis scale_color_viridis scale_fill_viridis
#' @importFrom withr with_seed
#'
NULL

#' Map values to distances from the center and find the frequency of these
#' distances
#'
#' This function interprets values as distances from a center (high
#' values = low distances) and calculates the frequencies of these distances.
#'
#' @details Used later to draw concentric circles with the frequencies
#' representing the number of points on a circle of the same radius.
#'
#' @param valuesDF A data frame with names on the first column and
#' positive integers on the second column.
#'
#' @return A data frame of distance frequencies.
#'
#' @keywords internal
#'
distFreq <- function(valuesDF){
    df <- dplyr::count(valuesDF, valuesDF[, 2])
    if (nrow(df) == 1)
        center <- df[1, 1] else{
            df <- df[order(df[, 1], decreasing=TRUE), ]
            if (df[2, 1] != df[1, 1])
                center <- df[1, 1] else
                    center <- df[1, 1]
        }
    colnames(df) <- c('Dist', 'Freq')
    df$Dist <- center - df$Dist
    return(df)
}

#' Generate the coordinates of points on a circle centered at origin
#'
#' This function generates nPoints on a circle of radius r
#' centered at origin.
#'
#' @param r Radius.
#' @param nPoints Number of points.
#' @param seed Random seed.
#'
#' @return A data frame with the coordinates of the points.
#'
#' @keywords internal
#'
pointsOnCircle <- function(r, nPoints, seed = 50){
    angleOffset <- with_seed(seed, runif(n=1, min=0, max=2 * pi))
    theta <- 2 * pi / nPoints
    points <- lapply(seq(nPoints),
                     function(k) c(r * cos(k * theta + angleOffset),
                                   r * sin(k * theta + angleOffset)))
    res <- do.call(rbind, points)
    colnames(res) <- c('x', 'y')
    return(res)
}

#' Compute the coordinates of items plotted on the figure made from concentric
#' circles
#'
#' This function computes the coordinates of items on the figure made from
#' concentric circles.
#'
#' @details A wrapper around \code{distFreq} and \code{pointsOnCircle}.
#'
#' @inheritParams distFreq
#' @inheritParams pointsOnCircle
#'
#' @return A data frame containing the coordinates of the items.
#'
#' @noRd
#'
itemCoords <- function(valuesDF, seed = 50){
    valuesDF <- valuesDF[order(valuesDF[, 2], decreasing=TRUE), ]
    distFreqDF <- distFreq(valuesDF)
    circlePoints <- do.call(rbind, lapply(seq_len(nrow(distFreqDF)), function(i)
        pointsOnCircle(distFreqDF$Dist[i], distFreqDF$Freq[i], seed + i)))
    df <- cbind(valuesDF[, 1, drop=FALSE],
                circlePoints,
                valuesDF[, c(2, 3)])
    df[, 5] <- as.factor(df[, 5])
    return(df)
}

#' Store the radii of the circles and the corresponding values
#'
#' This function stores the radii of the circles and the corresponding value
#' for each circle.
#'
#' @param itemCoordsDF Data frame wih item coordinates.
#' @param extraCircles Number of circles drawn beyond those required to include
#' the points.
#'
#' @return A data frame containing the radius and the corresponding value for
#' each circle.
#'
#' @keywords internal
#'
circleCoords <- function(itemCoordsDF, extraCircles = 0){
    values <- itemCoordsDF[, 4]
    minValue <- values[length(values)] - extraCircles
    maxValue <- values[1]
    nCircles <- maxValue - minValue + 1
    hasSharedMax <- 0
    if(length(values) > 1)
        hasSharedMax <- values[1] == values[2]
    df <- data.frame(
        x = rep(0, nCircles),
        y = rep(0, nCircles),
        r = seq(nCircles + hasSharedMax - 0.5, hasSharedMax + 0.5, -1),
        Value = seq(minValue, maxValue))
    return(df)
}

#' Draw radial plot for a data frame with positive integer-valued points
#'
#' This function draws a radial plot for a data frame, plotting positive
#' integer-valued points over concentric circles, with points located
#' more centrally representing higher values.
#'
#' @inheritParams distFreq
#' @inheritParams documentFun
#' @inheritParams circleCoords
#' @inheritParams pointsOnCircle
#' @inheritParams labelPoints
#' @param valueLegendTitle Legend title corresponding to the positive integer
#' column.
#' @param groupLegendTitle Legend title corresponding to the categorical
#' column.
#' @param breakDensity Factor used in calculating the number of breaks for
#' the values legend. Higher values of this argument add more breaks to the
#' legend, but no breaks at a distance below 1 will be allowed.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' valuesDF <- data.frame(Protein = paste0('P', seq(20)),
#' Value = sample(10, 20, replace=TRUE),
#' Group = sample(3, 20, replace=TRUE))
#' radialPlot(valuesDF, groupLegendTitle='Group')
#'
#' @export
#'
radialPlot <- function(valuesDF,
                       title = NULL,
                       valueLegendTitle = 'Value',
                       groupLegendTitle = NULL,
                       extraCircles = 0,
                       palette = rpColors(length(unique(valuesDF[, 3]))),
                       labelSize = 3,
                       pointSize = 0.8,
                       legendTitleSize = 10,
                       legendTextSize = 10,
                       labelRepulsion = 1,
                       labelPull = 0,
                       maxOverlaps = 15,
                       breakDensity = 6,
                       seed = 50,
                       ...){

    itemCoordsDF <- itemCoords(valuesDF, seed)
    circleCoordsDF <- circleCoords(itemCoordsDF, extraCircles)
    legendStep <- as.integer(itemCoordsDF[1, 4] / breakDensity) + 1
    p <- ggplot() +
        geom_circle(aes(x0=.data[['x']],
                        y0=.data[['y']],
                        r=.data[['r']],
                        fill=.data[['Value']],
                        color=.data[['Value']]),
                    data=circleCoordsDF) +
        scale_fill_viridis(option='viridis', begin=0.4,
                           breaks=seq(itemCoordsDF[1, 4], 1, -legendStep)) +
        scale_color_viridis(option='viridis', begin=0.4,
                            breaks=seq(itemCoordsDF[1, 4], 1, -legendStep),
                            guide='none') +
        labs(fill=valueLegendTitle) +
        theme_classic() + easy_remove_axes() + coord_fixed() +
        theme(plot.margin=margin(0, 0, 0, 0),
              legend.title=element_text(size=legendTitleSize),
              legend.text=element_text(size=legendTextSize)) +
        geom_text_repel(aes(x=.data[['x']],
                            y=.data[['y']],
                            label=.data[[names(itemCoordsDF)[1]]]),
                        data=itemCoordsDF, size=labelSize,
                        force=labelRepulsion,
                        force_pull=labelPull,
                        max.overlaps=maxOverlaps)
    if (!is.null(groupLegendTitle))
        p <- p + new_scale_color() +
        new_scale_fill() +
        geom_point(aes(x=.data[['x']],
                       y=.data[['y']],
                       color=.data[[names(itemCoordsDF)[5]]]),
                   data=itemCoordsDF,
                   size=pointSize) +
        scale_color_discrete(type=palette) +
        labs(color=groupLegendTitle) else
            p <- p + geom_point(aes(x=.data[['x']],
                                    y=.data[['y']]),
                                data=itemCoordsDF,
                                color=palette[1],
                                size=pointSize)
    p <- centerTitle(p, title, ...)
    return(p)
}

#'@importFrom ggalluvial geom_alluvium geom_stratum StatStratum
#'
NULL

#' Create an alluvial plot
#'
#' This function creates an alluvial plot.
#'
#' @inheritParams documentFun
#' @param df A data frame with two categorical columns and a numeric column.
#' @param fillColIndex Index of column used for coloring the alluvia.
#' @param curveType Curve type.
#' @param alpha Opaqueness level for the colors of the alluvia.
#' @param strataFill Color used for the strata.
#' @param labelSize Size of labels of strata elements.
#' @param ... Other arguments passed to \code{centerTitle}.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' df <- data.frame(x = sample(c('a','b', 'c', 'd', 'e', 'f'), 20,
#' replace=TRUE),
#' y = sample(c('p','q', 'r', 's', 't', 'u', 'v', 'w'), 20,
#' replace=TRUE),
#' z = runif(20, 1, 3))
#' riverPlot(df)
#'
#' @export
#'
riverPlot <- function(df,
                      title = NULL,
                      fillColIndex = 2,
                      curveType = 'sigmoid',
                      alpha = 0.8,
                      strataFill = 'lightgoldenrod1',
                      labelSize = 3,
                      axisTextSize = 12,
                      axisTitleSize = 12,
                      viridisPal = 'turbo',
                      ...){
    p <- ggplot(data=df, aes(axis1=.data[[names(df)[1]]],
                             axis2=.data[[names(df)[2]]],
                             y=.data[[names(df)[3]]])) +
        geom_alluvium(aes(fill=.data[[names(df)[fillColIndex]]]),
                      curve_type=curveType,
                      alpha=alpha) +
        geom_stratum(fill=strataFill) +
        geom_text(stat=StatStratum, aes(label=after_stat(stratum)),
                  size=labelSize) +
        scale_fill_viridis_d(option=viridisPal) +
        theme_void() +
        theme(legend.position='none')
    p <- centerTitle(p, title, ...)
    return(p)
}

#' Sort a data frame by the first column and convert the second to a factor
#'
#' This function sort a data frame by the first column and convert the second
#' to a factor.
#'
#' @param df A data frame.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' df <- data.frame(a = c(2, 4, 1, 3, 6),
#' b = c(2, 8, 3, 19, 3))
#' reorderDF(df)
#'
#' @export
#'
reorderDF <- function(df){
    df <- df[order(df[, 2], decreasing=TRUE), ]
    df <- df[order(df[, 1]), ]
    df[, 2] <- factor(df[, 2], levels=unique(df[, 2]))
    return(df)
}

#' Plot a numeric matrix or data frame
#'
#' This function plots a numeric matrix or data frame.
#'
#' @inheritParams documentFun
#' @param mat A numeric matrix or data frame.
#' @param doMelt Whether the input needs to be melted using
#' \code{reshape2::melt}.
#' @param showNumbers Whether to show a numeric value for each matrix element.
#' @param sigDigits Number of significant digits to be displayed for each
#' matrix element.
#' @param limits Limits of the color scale. If \code{NULL}, they will be taken
#' as the minimum and maximum of the values in the input object.
#' @param tileBoundaryColor Tile boundary color.
#' @param tileBoundaryWidth Tile boundary width.
#' @param reverseColors Whether to reverse the order of colors in the palette.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' mat <- matrix(round(runif(100, 0, 1), 2), nrow=10)
#' rownames(mat) <- paste0('R', seq(10))
#' colnames(mat) <- paste0('C', seq(10))
#' tilePlot(mat)
#'
#' @export
#'
tilePlot <- function(mat,
                     title = NULL,
                     xLab = NULL,
                     yLab = NULL,
                     doMelt = TRUE,
                     legendTitle = 'Value',
                     palette = 'Spectral',
                     reverseColors = TRUE,
                     showNumbers = TRUE,
                     sigDigits = 2,
                     limits = NULL,
                     labelSize = 3,
                     labelColor = 'black',
                     legendTextSize = 10,
                     legendTitleSize = 10,
                     axisTextSize = 12,
                     axisTitleSize = 12,
                     tileBoundaryColor = 'white',
                     tileBoundaryWidth = 0.2,
                     xAngle = 45,
                     vJust = 1,
                     hJust = 1,
                     ...){

    if(doMelt){
        if (is(mat)[1] != 'matrix')
            mat <- as.matrix(mat)

        mat <- round(mat, sigDigits)
        df <- reshape2::melt(mat)
    } else
        df <- mat

    palColors <- hcl.colors(50, palette)
    if (reverseColors)
         palColors <- rev(palColors)

    if(is.null(limits))
        limits <- c(min(df[, 3]), max(df[, 3]))

    df <- reorderDF(df)

    p <- ggplot(df, aes(x=.data[[names(df)[1]]],
                        y=.data[[names(df)[2]]],
                        fill=.data[[names(df)[3]]])) +
        geom_tile(color=tileBoundaryColor, lwd=tileBoundaryWidth) +
        theme_classic() +
        theme(legend.text=element_text(size=legendTextSize),
              legend.title=element_text(size=legendTitleSize),
              axis.text.x=element_text(angle=xAngle, vjust=vJust, hjust=hJust),
              axis.text=element_text(size=axisTextSize),
              axis.title=element_text(size=axisTitleSize)) +
        scale_fill_gradientn(colors=palColors, limits=limits) +
        labs(x=xLab, y=yLab, fill=legendTitle)

    if (showNumbers)
        p <- p + geom_text(aes(label=.data[[names(df)[3]]]),
                          color=labelColor,
                          size=labelSize)

    p <- centerTitle(p, title, ...)
    return(p)
}

#' Plot a correlation matrix
#'
#' This function plots a correlation matrix.
#'
#' @details A thin wrapper around \code{tilePlot}.
#'
#' @inheritParams tilePlot
#' @param ... Additional parameters passed to tilePlot.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' mat <- matrix(runif(100, -1, 1), nrow=10)
#' colnames(mat) <- paste0('I', seq(10))
#' mat <- round(cor(mat), 2)
#' correlationPlot(mat)
#'
#' @export
#'
correlationPlot <- function(mat, title = NULL,
                            legendTitle = 'Correlation', ...)
    return(tilePlot(mat, title, legendTitle=legendTitle, limits=c(-1, 1), ...))

#' @importFrom liver minmax
#' @importFrom reshape2 melt
#'
NULL

#' Create a rank summary
#'
#' This function creates a summary of multiple ranks provided for input items.
#'
#' @param df A data frame with ranks as columns and items as rows.
#'
#' @return A rank summary data frame with three columns: 'Rank', 'Item' and
#' 'Count'.
#'
#' @examples
#' df <- do.call(cbind, lapply(seq(30), function(i) sample(10, 10)))
#' rownames(df) <- paste0('M', seq(10))
#' colnames(df) <- paste0('R', seq(30))
#' rankSummary(df)
#'
#' @export
#'
rankSummary <- function(df){
    minPos <- min(df)
    maxPos <- max(df)
    positions <- sort(unique(as.numeric(df)))
    posDF <- do.call(rbind, lapply(positions, function(pos)
        apply(df, 1, function(x) sum(x == pos))))
    rownames(posDF) <- positions
    smr <- reshape2::melt(posDF)
    colnames(smr) <- c('Rank', 'Item', 'Count')
    smr <- smr[order(smr$Rank, -smr$Count), ]
    smr$Rank <- factor(smr$Rank)
    return(smr)
}

#' Compute the average rank of each item
#'
#' This function computes the average rank of each item.
#'
#' @param rankDF Rank data frame created with \code{rankSummary}.
#' @param sigDigits Number of significant digits to use for the mean ranks.
#'
#' @return A single-column data frame of average ranks.
#'
#' @noRd
#'
computeMeanRanks <- function(rankDF, sigDigits=2){
    if (is.null(sigDigits))
        sigDigits <- 2
    nMetrics <- sum(rankDF[, 3]) / length(unique(rankDF[, 2]))
    meanRanks <- sort(vapply(as.character(unique(rankDF[, 2])), function(x){
        subRankDF <- rankDF[rankDF[, 2] == x, ]
        return(round(sum(as.numeric(as.character(subRankDF[, 1])) *
                             subRankDF[, 3]) / nMetrics, sigDigits))},
        numeric(1)))
    return(data.frame(Item = names(meanRanks), MeanRank = meanRanks))
}

#' Create a rank plot
#'
#' This function creates a rank plot.
#'
#' @inheritParams documentFun
#' @param df A data frame with ranks as columns and items as rows, or a
#' summary data frame generated with \code{rankSummary}. If the latter,
#' \code{summarize} must be set to \code{FALSE}.
#' @param summarize Whether to summarize the ranks with \code{rankSummary}.
#' Must be set to \code{FALSE} if the input data frame has been generated with
#' \code{rankSummary}.
#' @param xLab Label of x axis.
#' @param sigDigits Number of significant digits used when displaying mean
#' ranks. If \code{NULL}, the mean ranks will not be displayed.
#' @param labelSize Size of label marking average rank for each item.
#' Ignored if \code{sigDigits} is \code{NULL}.
#' @param labelColor Color of label marking average rank for each item.
#' Ignored if \code{sigDigits} is \code{NULL}.
#' @param labelFace Font face of label marking average rank for each item. Must
#' be one among 'plain', 'bold', 'italic' and 'bold-italic'.
#' Ignored if \code{sigDigits} is \code{NULL}.
#' @param labelScalingFactor Scaling factor used when displaying mean ranks.
#' Ignored if \code{sigDigits} is \code{NULL}.
#' @param labelOffset Vertical offset used when displaying mean ranks.
#' Ignored if \code{sigDigits} is \code{NULL}.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' df <- do.call(cbind, lapply(seq(30), function(i) sample(10, 10)))
#' rownames(df) <- paste0('M', seq(10))
#' colnames(df) <- paste0('R', seq(30))
#' rankPlot(df)
#'
#' @export
#'
rankPlot <- function(df,
                     title = NULL,
                     summarize = TRUE,
                     viridisPal = 'turbo',
                     xLab = 'Item',
                     yLab = 'Rank count',
                     legendTitle = 'Rank',
                     sigDigits = NULL,
                     labelSize = 2.5,
                     labelColor = 'black',
                     labelFace = c('plain', 'bold', 'italic', 'bold-italic'),
                     legendTextSize = 10,
                     legendTitleSize = 10,
                     axisTextSize = 12,
                     axisTitleSize = 12,
                     xAngle = 45,
                     vJust = 1,
                     hJust = 1,
                     labelScalingFactor = 0.9,
                     labelOffset = 0.05,
                     ...){

    if(summarize)
        df <- rankSummary(df)

    meanRanks <- computeMeanRanks(df, sigDigits)
    itemOrder <- rownames(meanRanks)
    df[, 2] <- factor(df[, 2], levels=itemOrder)

    p <- ggplot() +
        geom_bar(aes(x=.data[[names(df)[2]]],
                     y=.data[[names(df)[3]]],
                     fill=.data[[names(df)[1]]]), df, stat='identity') +
        theme_classic() +
        scale_fill_viridis_d(option=viridisPal) + labs(x=xLab,
                                                       y=yLab,
                                                       fill=legendTitle) +
        theme(legend.text=element_text(size=legendTextSize),
              legend.title=element_text(size=legendTitleSize),
              axis.text.x=element_text(angle=xAngle, vjust=vJust, hjust=hJust),
              axis.text=element_text(size=axisTextSize),
              axis.title=element_text(size=axisTitleSize))

    if(!is.null(sigDigits)){
        labelFace <- match.arg(labelFace,
                               c('plain', 'bold', 'italic', 'bold-italic'))
        nRanks <- sum(df[, 3]) / length(unique(df[, 1]))
        labelHeights <- labelScalingFactor * nRanks *
            (liver::minmax(meanRanks[, 2]) + labelOffset)
        p <- p + geom_label(data=meanRanks,
                            aes(x=.data[[names(meanRanks)[1]]],
                               y=labelHeights,
                               label=.data[[names(meanRanks)[2]]]),
                            size=labelSize,
                            color=labelColor,
                            fontface=labelFace)
    }

    p <- centerTitle(p, title, ...)
    return(p)
}

#' Create a volcano plot
#'
#' This function creates a volcano plot for a data frame with a log column and
#' a p-value column. The gene names must be provided as row names.
#'
#' @details Users can input labeled genes in two ways:
#'
#' 1) By using p-value and log fold-change thresholds (the default option).
#'
#' 2) By inputting a list of labels.
#'
#' @inheritParams documentFun
#' @param df A data frame with rownames as genes, a log column and a
#' p-value column.
#' @param logCol Log column.
#' @param pvalCol P-value column.
#' @param logFCThr Threshold used to separate significant log values.
#' @param pvalThr Threshold used to separate significant p-values.
#' @param labeledGenes Gene labels to be displayed on the plot. Default is
#' \code{NULL}, entailing that gene labels will be displayed on the basis
#' of \code{labLogFCThr} and \code{labPvalThr}, if at least one of
#' them is not \code{NULL}.
#' @param labelOutside Display labels for points specified by
#' \code{labeledGenes} even if they fall outside the boundaries imposed by
#' \code{labLogFCThr} and \code{labPvalThr}. Ignored if \code{labeledGenes} is
#' \code{NULL} or both \code{labLogFCThr} and \code{labPvalThr} are \code{NULL}.
#' @param labLogFCThr Threshold used to plot gene labels based on log values.
#' Ignored if \code{labeledGenes} is not \code{NULL}.
#' @param labPvalThr Threshold used to plot gene labels based on p-values.
#' Ignored if \code{labeledGenes} is not \code{NULL}.
#' @param alpha Opaqueness level of point color.
#' @param theme Plot theme. Choose between 'bw', 'classic', 'linedraw'
#' and 'minimal'. Default is 'minimal'.
#' @param ... Additional arguments passed to
#' \code{EnhancedVolcano::EnhancedVolcano}.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' if (requireNamespace("EnhancedVolcano", quietly=TRUE)){
#' filePath <- system.file('extdata', 'volcanoPlot.qs2', package='henna')
#' df <- qs2::qs_read(filePath)
#' p <- volcanoPlot(df, title='Volcano plot - beta cells', pvalThr=1e-10,
#' logFCThr=1,
#' labPvalThr=1e-150,
#' labLogFCThr=5.3)
#' }
#'
#' @export
#'
volcanoPlot <- function(df,
                        title = NULL,
                        logCol = 'avg_log2FC',
                        pvalCol = 'p_val_adj',
                        xLab = expression(log[2] ~ fold ~ change),
                        yLab = expression(-log[10] ~ `p-value`),
                        legendTitle = 'Significance',
                        legendLabs = c('Not significant',
                                       expression(log[2] ~ FC),
                                       'p-value',
                                       expression(`p-value` ~ and
                                                  ~ log[2] ~ FC)),
                        legendPos = c('right', 'top', 'left', 'bottom'),
                        logFCThr = 1,
                        pvalThr = 1e-05,
                        labeledGenes = NULL,
                        labelOutside = FALSE,
                        labLogFCThr = 1.8,
                        labPvalThr = 1e-12,
                        labelType = c('boxed', 'free'),
                        labelSize = 2.2,
                        labelColor = 'black',
                        labelRepulsion = 1,
                        labelPull = 0,
                        maxOverlaps = 100,
                        boxPadding = 0.2,
                        labelPadding = 0.1,
                        labelSegWidth = 0.4,
                        pointSize = 0.8,
                        alpha = 0.6,
                        palette = c("gray31", "goldenrod2", "orchid3",
                                    "red2"),
                        legendTextSize = 10,
                        legendTitleSize = 10,
                        axisTextSize = 12,
                        axisTitleSize = 12,
                        theme = c('minimal', 'bw', 'classic', 'linedraw'),
                        ...){

    if (!requireNamespace("EnhancedVolcano", quietly=TRUE))
        stop("You need to install the EnhancedVolcano package from Bioconductor
             to use volcanoPlot.")
    legendPos <- match.arg(legendPos, c('right', 'top', 'left', 'bottom'))
    theme <- match.arg(theme, c('minimal', 'bw', 'classic', 'linedraw'))
    labelType <- match.arg(labelType, c('boxed', 'free'))

    noGenes <- paste0(rep('#', max(vapply(rownames(df), nchar, integer(1)))
                          + 1), collapse='')

    p <- EnhancedVolcano::EnhancedVolcano(df,
                                          lab=rownames(df),
                                          x=logCol,
                                          y=pvalCol,
                                          title=NULL,
                                          xlab=xLab,
                                          ylab=yLab,
                                          subtitle=NULL,
                                          caption=NULL,
                                          pCutoff=pvalThr,
                                          FCcutoff=logFCThr,
                                          selectLab=noGenes,
                                          labSize=labelSize,
                                          pointSize=pointSize,
                                          legendLabels=legendLabs,
                                          col=palette,
                                          colAlpha=alpha,
                                          ...) +
        labs(color=legendTitle)

    labelDF <- createLabelDFVolcano(df, logCol, pvalCol, labeledGenes,
                                    labelOutside, labLogFCThr, labPvalThr)

    if(!is.null(labelDF))
        p <- labelPoints(p, labelDF, rownames(labelDF), labelType, labelSize,
                         labelColor, labelRepulsion, labelPull, maxOverlaps,
                         boxPadding,labelPadding, labelSegWidth)
    p <- p + eval(as.name(paste0('theme_', theme)))()
    p <- p + theme(legend.position=legendPos,
                   legend.text=element_text(size=legendTextSize),
                   legend.title=element_text(size=legendTitleSize),
                   axis.text=element_text(size=axisTextSize),
                   axis.title=element_text(size=axisTitleSize))
    p <- centerTitle(p, title)
    return(p)
}

#' Internal function used for documenting other functions
#'
#' This function is used internally to help document other functions.
#'
#' @param title Plot title.
#' @param xLab x axis label.
#' @param yLab y axis label.
#' @param legendTitle Legend title.
#' @param legendLabs Legend labels.
#' @param legendPos Legend position.
#' @param palette Color palette.
#' @param viridisPal Viridis palette.
#' @param labeledPoints Point labels to be displayed on the plot.
#' @param labelType Whether to draw a box around labels (option 'boxed') or not
#' (option 'free'). Default is 'free'.
#' @param labelSize Label size.
#' @param labelColor Label color.
#' @param labelRepulsion Repulsion strength between labels.
#' @param labelPull Attraction strength between a text label
#' and its data point.
#' @param maxOverlaps Maximum number of allowed overlaps.
#' @param boxPadding Amount of padding around box.
#' @param labelPadding Amount of padding around label.
#' @param labelSegWidth Thickness of segment connecting label to point.
#' @param pointSize Point size.
#' @param pointShape Point shape.
#' @param legendTitleSize Legend title size.
#' @param legendTextSize Legend text size.
#' @param axisTextSize Axis text size.
#' @param axisTitleSize Axis title size.
#' @param xAngle Angle of x axis text.
#' @param vJust Vertical justification in [0, 1].
#' @param hJust Horizontal justification in [0, 1].
#' @param margins Plot margins. Must be a vector of size 4 listing the desired
#' top, right, bottom and left margin, in that order.
#' @param theme Plot theme.
#' @param pvalOffset Offset added to p-values to avoid infinite values when
#' taking logarithms.
#' @param ... Additional arguments passed to \code{centerTitle}.
#'
#' @return \code{NULL}. This function is only used internally for
#' documentation.
#'
#' @keywords internal

documentFun <- function(title = NULL,
                        xLab = 'x',
                        yLab = 'y',
                        legendTitle = 'Legend',
                        legendLabs = c('a', 'b'),
                        legendPos = 'right',
                        palette = 'Spectral',
                        viridisPal = 'turbo',
                        labeledPoints = NULL,
                        labelType = 'free',
                        labelOutside = TRUE,
                        labelSize = 2.5,
                        labelColor ='black',
                        labelRepulsion = 1,
                        labelPull = 1,
                        maxOverlaps = 50,
                        boxPadding = 0.2,
                        labelPadding = 0.1,
                        labelSegWidth = 0.4,
                        pointSize = 0.8,
                        pointShape = 1,
                        legendTitleSize = 10,
                        legendTextSize = 10,
                        axisTextSize = 12,
                        axisTitleSize = 12,
                        xAngle = 45,
                        vJust = 1,
                        hJust = 1,
                        margins = margin(0, -10, -10, -10),
                        theme = 'linedraw',
                        pvalOffset = 1e-317,
                        ...){}

#' @importFrom methods is
#'
NULL

#' Create segments between points and their nearest neighbors.
#'
#' This function creates segments between points and their nearest neighbors.
#'
#' @param df A data frame with two columns representing the \code{x}
#' and \code{y} coordinates of the points.
#'
#' @return A data frame with four columns representing the segments between
#' each point and its nearest neighbor.
#'
#' @noRd
#'
createPairSegments <- function(df){
    df <- addNNCol(df)
    segmentsDF <- cbind(df[, c(1, 2)], df[df$nn, c(1, 2)])
    colnames(segmentsDF) <- c('x', 'y', 'xend', 'yend')
    return(segmentsDF)
}

#' Create density plot
#'
#' This function creates a density plot.
#'
#' @inheritParams documentFun
#' @inheritParams labelPoints
#' @param df A data frame with at least two columns, representing the \code{x}
#' and \code{y} coordinates of the points. A score column can also be provided
#' as the third column. Nearest neighbor information can be provided in the last
#' column as a character vector with elements selected from the rownames.
#' @param colorScheme Color scheme. Choose between 'cake', 'cloudy', 'grapes',
#' 'lava', oasis', 'orichalc', 'sea', 'sky' and 'custom'. Default is 'cloudy'.
#' @param useSchemeDefaults Whether to use the default \code{segColor},
#' \code{pointColor} and \code{labelColor} values for scheme. Ignored
#' if \code{colorScheme} is set to 'custom'.
#' @param drawNN Whether to draw segments linking each point to its nearest
#' neighbor.
#' @param drawScores Whether to render scores on the plot. If set to
#' \code{TRUE}, the third column of the input data frame will be numeric and
#' scores will be taken from there.
#' @param palette A character vector of colors. Used only if color scheme is
#' set to 'custom'.
#' @param segColor Nearest neighbor segment color. Ignored if \code{drawNN} is
#' set to \code{FALSE}, or if \code{useSchemeDefaults} is \code{TRUE} and
#' \code{colorScheme} is different from 'custom'.
#' @param pointSize Point size.
#' @param pointColor Point color. Ignored if \code{useSchemeDefaults}
#' is \code{TRUE} and \code{colorScheme} is different from 'custom'.
#' @param segType Nearest neighbor segment type. Choose between 'solid',
#' 'dashed', 'dotted','dotdash', 'longdash' and 'twodash'. Ignored if
#' \code{drawNN} is set to \code{FALSE}.
#' @param segWidth Nearest neighbor segment width. Ignored if \code{drawNN} is
#' set to \code{FALSE}.
#' @param legendPos Legend position. Choose between 'right' and 'none'.
#' @param nGridPoints Number of grid points in each direction.
#' @param expandPerc Percentage by which the grid will be expanded.
#' @param labelColor Label color. Ignored if \code{useSchemeDefaults}
#' is \code{TRUE} and \code{colorScheme} is different from 'custom'.
#' @param verbose Whether output should be verbose.
#'
#' @return An object of class \code{gg}.
#'
#' @examples
#' x <- c(1, 2, 3, 4, 6, 7, 8, 10, 12, 11, 3, 6, 4, 1, 13, 13, 14, 18, 16)
#' y <- c(1, 3, 1, 4, 3, 2, 8, 2, 1, 11, 8, 8, 10, 14, 13, 11, 11, 12,15)
#' z <- round(runif(19, 75, 100), 2)
#' df <- data.frame(x, y, z)
#' rownames(df) <- paste0('p', rownames(df))
#' densityPlot(df)
#'
#' @export
#'
densityPlot <- function(df,
                        title = NULL,
                        colorScheme = c('cloudy', 'cake', 'grapes', 'lava',
                                        'oasis', 'orichalc', 'sea', 'sky',
                                        'custom'),
                        useSchemeDefaults = TRUE,
                        drawNN = TRUE,
                        drawScores = FALSE,
                        xLab = NULL,
                        yLab = NULL,
                        legendTitle = 'Density',
                        palette = NULL,
                        segColor = 'black',
                        pointSize = 1,
                        pointColor = 'red',
                        segType = c('dashed','solid', 'dotted',
                                    'dotdash', 'longdash', 'twodash'),
                        segWidth = 0.4,
                        nGridPoints = 300,
                        expandPerc = 20,
                        labelType = c('free', 'boxed'),
                        labelSize = 3,
                        labelColor = 'black',
                        labelRepulsion = 1,
                        labelPull = 1,
                        maxOverlaps = 10,
                        labelPadding = 0,
                        boxPadding = 0,
                        labelSegWidth = 0.4,
                        legendPos = c('right', 'none'),
                        legendTextSize = 10,
                        legendTitleSize = 10,
                        axisTextSize = 12,
                        axisTitleSize = 12,
                        verbose = FALSE,
                        ...){

    colorScheme <- match.arg(colorScheme, c('cloudy', 'cake', 'grapes',
                                            'lava', 'oasis', 'orichalc',
                                            'sea', 'sky', 'custom'))
    segType <- match.arg(segType, c('dashed','solid', 'dotted',
                                    'dotdash', 'longdash', 'twodash'))
    labelType <- match.arg(labelType, c('free', 'boxed'))
    legendPos <- match.arg(legendPos, c('right', 'none'))

    if (colorScheme == 'custom'){
        if (is.null(palette))
            stop('The custom color scheme requires an input palette.')
        palette <- palette
    } else {
        palette <- dpColors(colorScheme)
        if (useSchemeDefaults){
            colorsDF <- lpsColors()
            labelColor <- colorsDF['label', colorScheme]
            pointColor <- colorsDF['point', colorScheme]
            segColor <- colorsDF['segment', colorScheme]
        }
    }

    pointLabs <- rownames(df)
    if(ncol(df) > 2)
        if(drawScores)
            if(is(df[, 3])[1] == 'numeric')
                pointLabs <- mapply(function(x, y) paste0(x, '\n', y),
                                       rownames(df), round(df[, 3], 2)) else
                                        warning('drawScores is set to TRUE but ',
                                                'no scores are available in ',
                                                'the third column. Scores will ',
                                                'not be plotted.')

    p <- ggplot(df, aes(x=.data[[names(df)[1]]], y=.data[[names(df)[2]]])) +
        stat_density_2d(aes(fill=after_stat(density)),
                        geom="raster",
                        contour=FALSE,
                        n=nGridPoints) +
        scale_fill_gradientn(colors=palette) + theme_classic() +
        expand_limits(x=expandRange(df[, 1], expandPerc),
                      y=expandRange(df[, 2], expandPerc)) +
        labs(x=xLab, y=yLab, fill=legendTitle) +
        theme(legend.position=legendPos,
              legend.text=element_text(size=legendTextSize),
              legend.title=element_text(size=legendTitleSize),
              axis.text=element_text(size=axisTextSize),
              axis.title=element_text(size=axisTitleSize))

    if(drawNN){
        lastCol <- df[, ncol(df)]
        if(is(lastCol)[1] != 'character'){
            if(verbose)
                message('Nearest neighbor information not provided.',
                        ' Will be computed.')
        } else{
            extraNames <- setdiff(lastCol, rownames(df))
            if (length(extraNames))
                warning(extraNames[1], ' found in the last column but not',
                        ' in row names. Nearest neighbor information will be',
                        ' computed to replace the last column.')
        }
        segmentsDF <- createPairSegments(df)
        p <- p + geom_segment(data=segmentsDF,
                              aes(x=.data[['x']],
                                  y=.data[['y']],
                                  xend=.data[['xend']],
                                  yend=.data[['yend']]),
                              color=segColor,
                              linetype=segType,
                              linewidth=segWidth)
    }

    p <- p + geom_point(size=pointSize, color=pointColor)
    p <- labelPoints(p, df, pointLabs, labelType, labelSize, labelColor,
                     labelRepulsion, labelPull, maxOverlaps,
                     labelPadding, boxPadding, labelSegWidth)

    p <- centerTitle(p, title, ...)
    return(p)
}

#' Create a palette designed for densityPlot
#'
#' This function returns a palette designed for \code{densityPlot}.
#'
#' @param palette One of 'lava', 'oasis', 'orichalc', 'sea' and 'sky'.
#' @return A character vector of colors.
#'
#' @examples
#' dpColors('sea')
#'
#' @export
#'
dpColors <- function(palette = c('cake', 'cloudy', 'grapes', 'lava', 'oasis',
                                 'orichalc', 'sea','sky')){
    palette <- match.arg(palette, c('cake', 'cloudy', 'grapes', 'lava', 'oasis',
                                    'orichalc', 'sea','sky'))
    palList <- list(cake = c('orangered4', 'brown4','tomato4',
                             'coral4', 'lightyellow', 'snow',
                             'mistyrose', 'red2', 'firebrick3'),
                    cloudy = c('white', 'snow', 'snow1', 'lightblue1',
                               'lightskyblue1', 'lightskyblue', 'skyblue1',
                               'deepskyblue', 'deepskyblue1'),
                    grapes = c('olivedrab1', 'darkolivegreen1', 'mistyrose1',
                               'lightpink','palevioletred', 'palevioletred4',
                               'orchid4', 'darkorchid4', 'purple4'),
                    lava = c('bisque4','bisque3','bisque2',
                              'bisque1','bisque','lightyellow',
                              'goldenrod1','red2','red3'),
                    oasis = c('navajowhite1','bisque1','wheat1',
                              'darkolivegreen1','chartreuse','green',
                              'cadetblue1', 'cyan2', 'deepskyblue'),
                    orichalc = c('dodgerblue3','dodgerblue2', 'deepskyblue',
                                 'cyan2', 'honeydew1', 'lightgoldenrodyellow',
                                 'lemonchiffon','wheat','red'),
                    sea = c('midnightblue','dodgerblue4','dodgerblue3',
                            'dodgerblue2','deepskyblue','cyan2',
                            'lightgoldenrodyellow','darkolivegreen1','green'),
                    sky = c('deepskyblue', 'deepskyblue1', 'skyblue1',
                            'lightskyblue','lightskyblue1', 'lightblue1',
                            'snow', 'snow1', 'snow2'))
    return(palList[[palette]])
}

#' Return default label, point and segment colors for the dpColors palettes
#'
#' This function returns default label, point and segment colors for
#' the \code{dpColors} palettes.
#'
#' @inheritParams dpColors
#' @return A data frame of colors.
#'
#' @noRd
#'
lpsColors <- function(){
    df <- data.frame(cake = c('black', 'blue', 'deepskyblue'),
                     cloudy = c('black', 'red', 'plum1'),
                     grapes = c('black', 'blue', 'deepskyblue'),
                     lava = c('black', 'black', 'dodgerblue3'),
                     oasis = c('black', 'red', 'mediumpurple'),
                     orichalc = c('black', 'black', 'lightpink1'),
                     sea = c('black', 'red', 'plum1'),
                     sky = c('black', 'red', 'thistle'))
    rownames(df) <- c('label', 'point', 'segment')
    return(df)
}

#' Create the default hullPlot palette
#'
#' This function returns the default palette used by \code{hullPlot}.
#'
#' @return A character vector of colors.
#'
#' @export
#'
hpColors <- function()
    return(c('gold','purple', 'blue', 'red'))


#' Create a palette designed to represent dots over a viridis background
#'
#' This function returns a 10-color palette used as the default
#' of \code{radialPlot}.
#'
#' @param nColors Number of colors.
#'
#' @return A character vector of colors.
#'
#' @export
#'
rpColors <- function(nColors = 10){
    colors <- c('red', 'purple1', 'olivedrab1','darkorange1',
                'lavender', 'thistle1','green1','violetred4',
                'goldenrod1', 'firebrick4')
    return(colors[seq(nColors)])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank_plot.R
\name{rankPlot}
\alias{rankPlot}
\title{Create a rank plot}
\usage{
rankPlot(
  df,
  title = NULL,
  summarize = TRUE,
  viridisPal = "turbo",
  xLab = "Item",
  yLab = "Rank count",
  legendTitle = "Rank",
  sigDigits = NULL,
  labelSize = 2.5,
  labelColor = "black",
  labelFace = c("plain", "bold", "italic", "bold-italic"),
  legendTextSize = 10,
  legendTitleSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  xAngle = 45,
  vJust = 1,
  hJust = 1,
  labelScalingFactor = 0.9,
  labelOffset = 0.05,
  ...
)
}
\arguments{
\item{df}{A data frame with ranks as columns and items as rows, or a
summary data frame generated with \code{rankSummary}. If the latter,
\code{summarize} must be set to \code{FALSE}.}

\item{title}{Plot title.}

\item{summarize}{Whether to summarize the ranks with \code{rankSummary}.
Must be set to \code{FALSE} if the input data frame has been generated with
\code{rankSummary}.}

\item{viridisPal}{Viridis palette.}

\item{xLab}{Label of x axis.}

\item{yLab}{y axis label.}

\item{legendTitle}{Legend title.}

\item{sigDigits}{Number of significant digits used when displaying mean
ranks. If \code{NULL}, the mean ranks will not be displayed.}

\item{labelSize}{Size of label marking average rank for each item.
Ignored if \code{sigDigits} is \code{NULL}.}

\item{labelColor}{Color of label marking average rank for each item.
Ignored if \code{sigDigits} is \code{NULL}.}

\item{labelFace}{Font face of label marking average rank for each item. Must
be one among 'plain', 'bold', 'italic' and 'bold-italic'.
Ignored if \code{sigDigits} is \code{NULL}.}

\item{legendTextSize}{Legend text size.}

\item{legendTitleSize}{Legend title size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{xAngle}{Angle of x axis text.}

\item{vJust}{Vertical justification in [0, 1].}

\item{hJust}{Horizontal justification in [0, 1].}

\item{labelScalingFactor}{Scaling factor used when displaying mean ranks.
Ignored if \code{sigDigits} is \code{NULL}.}

\item{labelOffset}{Vertical offset used when displaying mean ranks.
Ignored if \code{sigDigits} is \code{NULL}.}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function creates a rank plot.
}
\examples{
df <- do.call(cbind, lapply(seq(30), function(i) sample(10, 10)))
rownames(df) <- paste0('M', seq(10))
colnames(df) <- paste0('R', seq(30))
rankPlot(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{rpColors}
\alias{rpColors}
\title{Create a palette designed to represent dots over a viridis background}
\usage{
rpColors(nColors = 10)
}
\arguments{
\item{nColors}{Number of colors.}
}
\value{
A character vector of colors.
}
\description{
This function returns a 10-color palette used as the default
of \code{radialPlot}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{vertexComponents}
\alias{vertexComponents}
\title{Return the connected components of vertices}
\usage{
vertexComponents(df, colName = "component")
}
\arguments{
\item{df}{A data frame with two categorical columns representing graph edges
and a connected components column.}

\item{colName}{Name of the connected components column.}
}
\value{
A factor vector representing the connected component of each
vertex.
}
\description{
This function returns the connected components of vertices from a graph
data frame in which edges have been assigned connected components.
}
\examples{
df <- data.frame(gene1 = c('A', 'B', 'C', 'A'),
gene2 = c('B', 'D', 'F', 'G'),
component = c(1, 1, 2, 1))
vertexComponents(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radial_plot.R
\name{distFreq}
\alias{distFreq}
\title{Map values to distances from the center and find the frequency of these
distances}
\usage{
distFreq(valuesDF)
}
\arguments{
\item{valuesDF}{A data frame with names on the first column and
positive integers on the second column.}
}
\value{
A data frame of distance frequencies.
}
\description{
This function interprets values as distances from a center (high
values = low distances) and calculates the frequencies of these distances.
}
\details{
Used later to draw concentric circles with the frequencies
representing the number of points on a circle of the same radius.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radial_plot.R
\name{pointsOnCircle}
\alias{pointsOnCircle}
\title{Generate the coordinates of points on a circle centered at origin}
\usage{
pointsOnCircle(r, nPoints, seed = 50)
}
\arguments{
\item{r}{Radius.}

\item{nPoints}{Number of points.}

\item{seed}{Random seed.}
}
\value{
A data frame with the coordinates of the points.
}
\description{
This function generates nPoints on a circle of radius r
centered at origin.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tile_plot.R
\name{reorderDF}
\alias{reorderDF}
\title{Sort a data frame by the first column and convert the second to a factor}
\usage{
reorderDF(df)
}
\arguments{
\item{df}{A data frame.}
}
\value{
An object of class \code{gg}.
}
\description{
This function sort a data frame by the first column and convert the second
to a factor.
}
\examples{
df <- data.frame(a = c(2, 4, 1, 3, 6),
b = c(2, 8, 3, 19, 3))
reorderDF(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hull_plot.R
\name{quadBorders}
\alias{quadBorders}
\title{Find the coordinates of the points establishing the four divisions of
the hull}
\usage{
quadBorders(pointsDF, xInt, yInt, vCoords, hCoords)
}
\arguments{
\item{pointsDF}{A data frame with the x and y coordinates of the points.}

\item{xInt}{The coordinate where the vertical line intersects the x axis.}

\item{yInt}{The coordinate where the horizontal line intersects the y axis.}

\item{vCoords}{The y coordinates of the two points where the vertical line
intersects the convex hull.}

\item{hCoords}{The x coordinates of the two points where the horizontal line
intersects the convex hull.}
}
\value{
A data frame with 2 columns representing the 12 points (not unique)
determining the boundaries of the hull divisions.
}
\description{
This function finds the coordinates of the points establishing the
four divisions of the hull
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hull_plot.R
\name{borderCoords}
\alias{borderCoords}
\title{Find the coordinates where a vertical or horizontal line intersects the hull}
\usage{
borderCoords(df, axis, axisIntersect)
}
\arguments{
\item{df}{A four-column data frame representing segments.}

\item{axis}{An integer representing the axis intersected by the vertical or
horizontal line, x (1) or y (2).}

\item{axisIntersect}{The coordinate where the vertical or horizontal line
intersects the relevant axis.}
}
\value{
A vector of size two representing the coordinates of the two
intersection points between the vertical or horizontal line and the convex
hull on the axis different from the input axis.
}
\description{
This function finds the coordinates where a vertical or horizontal line
intersects the hull.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radial_plot.R
\name{circleCoords}
\alias{circleCoords}
\title{Store the radii of the circles and the corresponding values}
\usage{
circleCoords(itemCoordsDF, extraCircles = 0)
}
\arguments{
\item{itemCoordsDF}{Data frame wih item coordinates.}

\item{extraCircles}{Number of circles drawn beyond those required to include
the points.}
}
\value{
A data frame containing the radius and the corresponding value for
each circle.
}
\description{
This function stores the radii of the circles and the corresponding value
for each circle.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{connectedComponents}
\alias{connectedComponents}
\title{Find the connected components of a graph represented as a data frame}
\usage{
connectedComponents(df, colName = "component")
}
\arguments{
\item{df}{A data frame with two categorical columns representing the edges
of a graph.}

\item{colName}{Name of the connected components column to be added.}
}
\value{
A data frame with a column indicating the connected component of
each edge.
}
\description{
This function finds the connected components of a graph represented as
a data frame.
}
\examples{
df <- data.frame(
gene1 = paste0('G', c(1, 2, 6, 7, 8, 9,
11, 25, 32, 17, 18)),
gene2 = paste0('G', c(2, 8, 8, 8, 1, 25,
32, 24, 24, 26, 26))
)
connectedComponents(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{centerTitle}
\alias{centerTitle}
\title{Add a centered title to a plot}
\usage{
centerTitle(p, title, ...)
}
\arguments{
\item{p}{A ggplot object.}

\item{title}{Plot title.}

\item{...}{Other arguments passed to \code{ggplot2::element_text}.}
}
\value{
A ggplot object.
}
\description{
This function adds a centered title to a ggplot object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hull_plot.R
\name{splitInTwo}
\alias{splitInTwo}
\title{Split the convex hull in two along an input line}
\usage{
splitInTwo(
  p,
  pointsDF,
  axisIntersect,
  pointCoords,
  borderPoints,
  legendLabs = paste0("Group ", seq(2)),
  alpha = 0.5
)
}
\arguments{
\item{p}{A ggplot object representing the hull.}

\item{pointsDF}{A data frame with the x and y coordinates of the points.}

\item{axisIntersect}{The coordinate where the vertical or horizontal line
intersects the relevant axis.}

\item{pointCoords}{The coordinates of the input points on the axis
perpendicular to the input border line.}

\item{borderPoints}{The points where the border line intersects the
convex hull.}

\item{legendLabs}{Legend labels.}

\item{alpha}{Opaqueness level.}
}
\value{
A ggplot object showing the hull split in two parts along the input
line.
}
\description{
This function splits the convex hull in two along an input vertical or
horizontal line.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_plot.R
\name{networkPlotDF}
\alias{networkPlotDF}
\title{Prepare data frame for network plot}
\usage{
networkPlotDF(df, rankCol = "rank")
}
\arguments{
\item{df}{Data frame.}

\item{rankCol}{Name of the rank column.}
}
\value{
A data frame ready to serve as input to \code{networkPlot}.
}
\description{
This function prepares a data frame for network plot.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_points.R
\name{createLabelDFTemplate}
\alias{createLabelDFTemplate}
\title{Create a label data frame}
\usage{
createLabelDFTemplate(
  df,
  xCol = 1,
  yCol = 2,
  labeledPoints = NULL,
  labelOutside = FALSE,
  labXThr = NULL,
  labYThr = NULL,
  adjFunX = identity,
  adjFunY = identity,
  compFunX = `>=`,
  compFunY = `>=`
)
}
\arguments{
\item{df}{A data frame.}

\item{xCol}{Column representing the \code{x} axis.}

\item{yCol}{Column representing the \code{y} axis.}

\item{labeledPoints}{Point labels to be displayed on the plot. Default is
\code{NULL}, entailing that point labels will be displayed on the basis
of \code{labXThr} and \code{labYThr}, if at least one of them is not
\code{NULL}.}

\item{labelOutside}{Display labels for points specified by
\code{labeledPoints} even if they fall outside the boundaries imposed by
\code{labXThr} and \code{labYThr}. Ignored if \code{labeledPoints} is
\code{NULL} or both \code{labXThr} and \code{labYThr} are \code{NULL}.}

\item{labXThr}{Threshold used to plot labels based on \code{xCol} values.}

\item{labYThr}{Threshold used to plot labels based on \code{yCol} values.}

\item{adjFunX}{Function used to adjust \code{xCol} values. Default is
\code{identity} (no adjustment will be performed).}

\item{adjFunY}{Function used to adjust \code{yCol} values. Default is
\code{identity} (no adjustment will be performed).}

\item{compFunX}{Function used for the selection of \code{xCol} values
based on \code{labXThr}.}

\item{compFunY}{Function used for the selection of \code{yCol} values
based on \code{labYThr}.}
}
\value{
A label data frame.
}
\description{
This function creates a label data frame to be used by plotting functions
that label selected points.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dfEuclidean}
\alias{dfEuclidean}
\title{Find the Euclidean distance between two points in a data frame}
\usage{
dfEuclidean(df, i, j, pointCoords = c(1, 2), replaceZero = 0)
}
\arguments{
\item{df}{A data frame containing the coordinates
of the points.}

\item{i}{Row name or index.}

\item{j}{Row name or index}

\item{pointCoords}{The row names or indices of point coordinates.}

\item{replaceZero}{Value to replace zero with. If set to 0, no replacement
will occur.}
}
\value{
A numeric value.
}
\description{
This function finds the Euclidean distance between two points in a data
frame.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/class_plot.R
\name{classPlot}
\alias{classPlot}
\title{Plot bars for item counts grouped and colored by class}
\usage{
classPlot(
  df,
  title = NULL,
  xLab = "Value",
  yLab = "Item",
  legendTitle = "Class",
  palette = "Spectral",
  labelSize = 2.5,
  labelColor = "black",
  legendTitleSize = 10,
  legendTextSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  decreasing = TRUE,
  valueCutoff = 0,
  ...
)
}
\arguments{
\item{df}{A data frame with at least three columns. Its first column
(categorical) colors the plot bars. The second column (categorical)
labels the plots bars. The third column (numeric) sets the bar lengths.}

\item{title}{Plot title.}

\item{xLab}{x axis label.}

\item{yLab}{y axis label.}

\item{legendTitle}{Legend title.}

\item{palette}{Color palette.}

\item{labelSize}{Label size.}

\item{labelColor}{Label color.}

\item{legendTitleSize}{Legend title size.}

\item{legendTextSize}{Legend text size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{decreasing}{Whether to display the bars in decreasing order of length.}

\item{valueCutoff}{Cutoff used for filtering the input data frame based on
the third (value) column. Only values above this cutoff will be displayed on
the plot.}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function plots bars for item counts grouped and colored by class.
}
\examples{
 df <- data.frame(Class = sample(paste0('C', seq(13)), 25, replace=TRUE),
 Item = paste0('I', seq(25)),
 Value = runif(25, 0.5, 1))
 classPlot(df)

 df <- data.frame(Class = sample(paste0('C', seq(13)), 25, replace=TRUE),
 Item = sample(paste0('I', seq(21)), 25, replace=TRUE),
 Value = runif(25, 0.5, 1))
 classPlot(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/river_plot.R
\name{riverPlot}
\alias{riverPlot}
\title{Create an alluvial plot}
\usage{
riverPlot(
  df,
  title = NULL,
  fillColIndex = 2,
  curveType = "sigmoid",
  alpha = 0.8,
  strataFill = "lightgoldenrod1",
  labelSize = 3,
  axisTextSize = 12,
  axisTitleSize = 12,
  viridisPal = "turbo",
  ...
)
}
\arguments{
\item{df}{A data frame with two categorical columns and a numeric column.}

\item{title}{Plot title.}

\item{fillColIndex}{Index of column used for coloring the alluvia.}

\item{curveType}{Curve type.}

\item{alpha}{Opaqueness level for the colors of the alluvia.}

\item{strataFill}{Color used for the strata.}

\item{labelSize}{Size of labels of strata elements.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{viridisPal}{Viridis palette.}

\item{...}{Other arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function creates an alluvial plot.
}
\examples{
df <- data.frame(x = sample(c('a','b', 'c', 'd', 'e', 'f'), 20,
replace=TRUE),
y = sample(c('p','q', 'r', 's', 't', 'u', 'v', 'w'), 20,
replace=TRUE),
z = runif(20, 1, 3))
riverPlot(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{isPointOnBoundary}
\alias{isPointOnBoundary}
\title{Check if a point is on a polygon boundary}
\usage{
isPointOnBoundary(xPoint, yPoint, boundary)
}
\arguments{
\item{xPoint}{x coordinate of point P.}

\item{yPoint}{y coordinate of point P.}

\item{boundary}{A data frame with four columns representing segments
comprising the boundary.}
}
\value{
Logical; whether the point is on the boundary.
}
\description{
This function checks if a point P is on a polygon boundary.
}
\examples{
pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
12, 13, 15, 16),
y = c(1, 1, 2, 3, 3, 2,
1, 2, 1))

hullIndices <- grDevices::chull(pointsDF[, 1], pointsDF[, 2])
hull <- convexHull(pointsDF, hullIndices)
hullSegments <- pointsToSegments(hull)

isPointOnBoundary(2, 3, hullSegments)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/label_points.R
\name{labelPoints}
\alias{labelPoints}
\title{Label points in a ggplot object}
\usage{
labelPoints(
  p,
  labelDF,
  pointLabs = rownames(labelDF),
  labelType = c("free", "boxed"),
  labelSize = 2.5,
  labelColor = "black",
  labelRepulsion = 1,
  labelPull = 1,
  maxOverlaps = 50,
  boxPadding = 0.2,
  labelPadding = 0.1,
  labelSegWidth = 0.4,
  ...
)
}
\arguments{
\item{p}{A ggplot object.}

\item{labelDF}{Label data frame.}

\item{pointLabs}{Labels of points.}

\item{labelType}{Whether to draw a box around labels (option 'boxed') or not
(option 'free'). Default is 'free'.}

\item{labelSize}{Label size.}

\item{labelColor}{Label color.}

\item{labelRepulsion}{Repulsion strength between labels.}

\item{labelPull}{Attraction strength between a text label
and its data point.}

\item{maxOverlaps}{Maximum number of allowed overlaps.}

\item{boxPadding}{Amount of padding around box.}

\item{labelPadding}{Amount of padding around label.}

\item{labelSegWidth}{Thickness of segment connecting label to point.}

\item{...}{Additional arguments passed to \code{geom_text_repel}
(if \code{labelType} is 'free') or \code{geom_text_label}
(if \code{labelType} is 'boxed').}
}
\value{
A ggplot object.
}
\description{
This function labels points in a ggplot object.
}
\examples{
filePath <- system.file('extdata', 'hullPlot.qs2', package='henna')
sharedDF <- qs2::qs_read(filePath)
name1 <- 'alpha'
name2 <- 'delta'
legendLabs <- as.factor(c('Non-top',
'Shared',
paste0('Top only for ', name2),
paste0('Top only for ', name1)))
p <- hullPlot(sharedDF, 'Shared markers plot', xInt=1.5, yInt=1.3,
xLab=paste0('avg_log2FC (', name1, ')'),
yLab=paste0('avg_log2FC (', name2, ')'),
legendLabs=legendLabs)
labelDF <- sharedDF[sharedDF[, 'avg_log2FC_1'] > 1.5 &
sharedDF[, 'avg_log2FC_2'] > 1.3, ]
p <- labelPoints(p, labelDF, labelType='boxed', nudge_x=0.1, nudge_y=0.1)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank_plot.R
\name{rankSummary}
\alias{rankSummary}
\title{Create a rank summary}
\usage{
rankSummary(df)
}
\arguments{
\item{df}{A data frame with ranks as columns and items as rows.}
}
\value{
A rank summary data frame with three columns: 'Rank', 'Item' and
'Count'.
}
\description{
This function creates a summary of multiple ranks provided for input items.
}
\examples{
df <- do.call(cbind, lapply(seq(30), function(i) sample(10, 10)))
rownames(df) <- paste0('M', seq(10))
colnames(df) <- paste0('R', seq(30))
rankSummary(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hull_plot.R
\name{splitHull}
\alias{splitHull}
\title{Split the convex hull in four parts along two input lines}
\usage{
splitHull(
  p,
  pointsDF,
  hullSegments,
  xInt = NULL,
  yInt = NULL,
  lineColor = "navy",
  lineWidth = 0.3,
  lineType = c("dashed", "solid", "dotted", "dotdash", "longdash", "twodash"),
  legendLabs = paste0("Group ", seq(4)),
  alpha = 0.2
)
}
\arguments{
\item{p}{A ggplot object representing the hull.}

\item{pointsDF}{A data frame with the x and y coordinates of the points.}

\item{hullSegments}{Data frame of segments that define the convex hull.}

\item{xInt}{The coordinate where the vertical line intersects the x axis.}

\item{yInt}{The coordinate where the horizontal line intersects the y axis.}

\item{lineColor}{The color of the horizontal and vertical dividing lines,
if provided. If \code{NULL}, no dividing lines will be drawn, though the
hull will still be split along these lines if \code{xInt}
and/or \code{yInt}are not \code{NULL}.}

\item{lineWidth}{The width of the horizontal and vertical dividing lines.
Ignored if \code{lineColor} is \code{NULL}.}

\item{lineType}{The type of the horizontal and vertical dividing lines.
Choose between 'dashed','solid', 'dotted', 'dotdash', 'longdash' and
'twodash'. Default is 'dashed'. Ignored if \code{lineColor} is \code{NULL}.}

\item{legendLabs}{Legend labels.}

\item{alpha}{Opaqueness level.}
}
\value{
An object of class \code{gg} showing the hull split along the input
axes.
}
\description{
This function splits the convex hull in two parts along a vertical and a
horizontal line.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_plot.R
\name{networkPlot}
\alias{networkPlot}
\title{Plot graph with the option of using different colors for connected components}
\usage{
networkPlot(
  df,
  title = NULL,
  numCol = NULL,
  numColType = c("weights", "ranks"),
  nodeSize = 10,
  nodeTextSize = 2.3,
  palette = "grDevices::Spectral",
  nodeColor = NULL,
  edgeWidth = 1,
  edgeColor = "black",
  edgeScales = c(0.2, 0.6),
  ...
)
}
\arguments{
\item{df}{Data frame.}

\item{title}{Plot title.}

\item{numCol}{Name of the numeric column used to vary edge widths. If no
such column is provided, set to \code{NULL}.}

\item{numColType}{The type of the numeric column used to vary edge widths.
Choose between 'weights' and 'ranks' Ignored if \code{numCol}
is \code{NULL}.}

\item{nodeSize}{Size of graph nodes.}

\item{nodeTextSize}{Size of text on graph nodes.}

\item{palette}{grDevices palette used for coloring nodes.
Ignored if nodeColor is not \code{NULL}.}

\item{nodeColor}{Color used for nodes. Default is \code{NULL}
(\code{palette} will instead be used).}

\item{edgeWidth}{Width to be used for all edges before scaling
if \code{numCol} is \code{NULL}; otherwise ignored.}

\item{edgeColor}{Color used for edges.}

\item{edgeScales}{Edge width scales. Must be a numeric vector of size
2 (minimum and maximum).}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{ggraph}.
}
\description{
This function plots the graph of the data frame and optionally uses different
colors for nodes belonging to different connected components.
}
\examples{
df <- data.frame(gene1 = paste0('G', c(1, 2, 5, 6, 7, 17)),
gene2 = paste0('G', c(2, 5, 8, 11, 11, 11)),
rank = c(1, 1, 3, 3, 3, 3))
networkPlot(df, numCol='rank', numColType='ranks')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{dfPoint}
\alias{dfPoint}
\title{Extract point from data frame of points.}
\usage{
dfPoint(df, i, pointCoords = c(1, 2))
}
\arguments{
\item{df}{A data frame containing the coordinates
of the points.}

\item{i}{Row name or index.}

\item{pointCoords}{The row names or indices of point coordinates.}
}
\value{
A numeric vector containing the point coordinates.
}
\description{
This function extracts a point from a data frame of points.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tile_plot.R
\name{correlationPlot}
\alias{correlationPlot}
\title{Plot a correlation matrix}
\usage{
correlationPlot(mat, title = NULL, legendTitle = "Correlation", ...)
}
\arguments{
\item{mat}{A numeric matrix or data frame.}

\item{title}{Plot title.}

\item{legendTitle}{Legend title.}

\item{...}{Additional parameters passed to tilePlot.}
}
\value{
An object of class \code{gg}.
}
\description{
This function plots a correlation matrix.
}
\details{
A thin wrapper around \code{tilePlot}.
}
\examples{
mat <- matrix(runif(100, -1, 1), nrow=10)
colnames(mat) <- paste0('I', seq(10))
mat <- round(cor(mat), 2)
correlationPlot(mat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{pointsToSegments}
\alias{pointsToSegments}
\title{Construct a data frame of segments from a data frame of points}
\usage{
pointsToSegments(pointsDF, joinEnds = TRUE)
}
\arguments{
\item{pointsDF}{A data frame with the x and y coordinates of the points.
Each point must appear only once.}

\item{joinEnds}{Whether to join the last point with the first one.}
}
\value{
A data frame of segments represented using four columns
(\code{x}, \code{y}, \code{xEnd}, \code{yEnd}).
}
\description{
This function constructs a data frame of segments from a data frame of
points.
}
\examples{
pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
12, 13, 15, 16),
y = c(1, 1, 2, 3, 3, 2, 1, 2, 1))

hullIndices <- grDevices::chull(pointsDF[, 1], pointsDF[, 2])
hull <- convexHull(pointsDF, hullIndices)
pointsToSegments(hull)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/hull_plot.R
\name{hullPlot}
\alias{hullPlot}
\title{Plot the convex hull of a set of points}
\usage{
hullPlot(
  pointsDF,
  title = NULL,
  xInt = NULL,
  yInt = NULL,
  palette = hpColors(),
  lineColor = "navy",
  lineWidth = 0.3,
  lineType = c("dashed", "solid", "dotted", "dotdash", "longdash", "twodash"),
  hullWidth = 0,
  xLab = NULL,
  yLab = NULL,
  legendLabs = paste0("Group ", seq(4)),
  pointSize = 1,
  pointShape = 4,
  alpha = 0.2,
  labeledPoints = NULL,
  labelOutside = FALSE,
  labXThr = NULL,
  labYThr = NULL,
  labelType = c("free", "boxed"),
  labelSize = 2.5,
  labelColor = "black",
  labelRepulsion = 1,
  labelPull = 0,
  maxOverlaps = 10,
  legendPos = "bottom",
  legendTextSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  ...
)
}
\arguments{
\item{pointsDF}{A data frame with the x and y coordinates of the points.}

\item{title}{Plot title.}

\item{xInt}{The coordinate where the vertical line intersects the x axis.}

\item{yInt}{The coordinate where the horizontal line intersects the y axis.}

\item{palette}{Color palette.}

\item{lineColor}{The color of the horizontal and vertical dividing lines,
if provided. If \code{NULL}, no dividing lines will be drawn, though the
hull will still be split along these lines if \code{xInt}
and/or \code{yInt}are not \code{NULL}.}

\item{lineWidth}{The width of the horizontal and vertical dividing lines.
Ignored if \code{lineColor} is \code{NULL}.}

\item{lineType}{The type of the horizontal and vertical dividing lines.
Choose between 'dashed','solid', 'dotted', 'dotdash', 'longdash' and
'twodash'. Default is 'dashed'. Ignored if \code{lineColor} is \code{NULL}.}

\item{hullWidth}{Width of the convex hull. If 0 (as default), the convex
hull will not be displayed.}

\item{xLab}{x axis label.}

\item{yLab}{y axis label.}

\item{legendLabs}{Legend labels.}

\item{pointSize}{Point size.}

\item{pointShape}{Point shape.}

\item{alpha}{Opaqueness level.}

\item{labeledPoints}{Point labels to be displayed on the plot.}

\item{labelOutside}{Display labels for points specified by
\code{labeledPoints} even if they fall outside the boundaries imposed by
\code{labXThr} and \code{labYThr}. Ignored if \code{labeledPoints} is
\code{NULL} or both \code{labXThr} and \code{labYThr} are \code{NULL}.}

\item{labXThr}{Threshold used to plot labels based on \code{xCol} values.}

\item{labYThr}{Threshold used to plot labels based on \code{yCol} values.}

\item{labelType}{Whether to draw a box around labels (option 'boxed') or not
(option 'free'). Default is 'free'.}

\item{labelSize}{Label size. Ignored if \code{labelDF} is \code{NULL}.}

\item{labelColor}{Label color. Ignored if \code{labelDF} is \code{NULL}.}

\item{labelRepulsion}{Repulsion strength between labels.}

\item{labelPull}{Attraction strength between a text label
and its data point.}

\item{maxOverlaps}{Maximum overlaps. Ignored if \code{labelDF}
is \code{NULL}.}

\item{legendPos}{Legend position.}

\item{legendTextSize}{Legend text size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function plots the convex hull of a set of points. It can also draw
a vertical or a horizontal line (or both), dividing the hull into areas of
different colors.
}
\examples{
pointsDF <- data.frame(x = c(1, 2, 4, 7, 10,
12, 13, 15, 16),
y = c(1, 1, 2, 3, 3, 2,
1, 2, 1))
hullPlot(pointsDF, 'Hull plot', 7, 1.5)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tile_plot.R
\name{tilePlot}
\alias{tilePlot}
\title{Plot a numeric matrix or data frame}
\usage{
tilePlot(
  mat,
  title = NULL,
  xLab = NULL,
  yLab = NULL,
  doMelt = TRUE,
  legendTitle = "Value",
  palette = "Spectral",
  reverseColors = TRUE,
  showNumbers = TRUE,
  sigDigits = 2,
  limits = NULL,
  labelSize = 3,
  labelColor = "black",
  legendTextSize = 10,
  legendTitleSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  tileBoundaryColor = "white",
  tileBoundaryWidth = 0.2,
  xAngle = 45,
  vJust = 1,
  hJust = 1,
  ...
)
}
\arguments{
\item{mat}{A numeric matrix or data frame.}

\item{title}{Plot title.}

\item{xLab}{x axis label.}

\item{yLab}{y axis label.}

\item{doMelt}{Whether the input needs to be melted using
\code{reshape2::melt}.}

\item{legendTitle}{Legend title.}

\item{palette}{Color palette.}

\item{reverseColors}{Whether to reverse the order of colors in the palette.}

\item{showNumbers}{Whether to show a numeric value for each matrix element.}

\item{sigDigits}{Number of significant digits to be displayed for each
matrix element.}

\item{limits}{Limits of the color scale. If \code{NULL}, they will be taken
as the minimum and maximum of the values in the input object.}

\item{labelSize}{Label size.}

\item{labelColor}{Label color.}

\item{legendTextSize}{Legend text size.}

\item{legendTitleSize}{Legend title size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{tileBoundaryColor}{Tile boundary color.}

\item{tileBoundaryWidth}{Tile boundary width.}

\item{xAngle}{Angle of x axis text.}

\item{vJust}{Vertical justification in [0, 1].}

\item{hJust}{Horizontal justification in [0, 1].}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function plots a numeric matrix or data frame.
}
\examples{
mat <- matrix(round(runif(100, 0, 1), 2), nrow=10)
rownames(mat) <- paste0('R', seq(10))
colnames(mat) <- paste0('C', seq(10))
tilePlot(mat)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{hpColors}
\alias{hpColors}
\title{Create the default hullPlot palette}
\usage{
hpColors()
}
\value{
A character vector of colors.
}
\description{
This function returns the default palette used by \code{hullPlot}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/volcano_plot.R
\name{volcanoPlot}
\alias{volcanoPlot}
\title{Create a volcano plot}
\usage{
volcanoPlot(
  df,
  title = NULL,
  logCol = "avg_log2FC",
  pvalCol = "p_val_adj",
  xLab = expression(log[2] ~ fold ~ change),
  yLab = expression(-log[10] ~ `p-value`),
  legendTitle = "Significance",
  legendLabs = c("Not significant", expression(log[2] ~ FC), "p-value",
    expression(`p-value` ~ and ~ log[2] ~ FC)),
  legendPos = c("right", "top", "left", "bottom"),
  logFCThr = 1,
  pvalThr = 1e-05,
  labeledGenes = NULL,
  labelOutside = FALSE,
  labLogFCThr = 1.8,
  labPvalThr = 1e-12,
  labelType = c("boxed", "free"),
  labelSize = 2.2,
  labelColor = "black",
  labelRepulsion = 1,
  labelPull = 0,
  maxOverlaps = 100,
  boxPadding = 0.2,
  labelPadding = 0.1,
  labelSegWidth = 0.4,
  pointSize = 0.8,
  alpha = 0.6,
  palette = c("gray31", "goldenrod2", "orchid3", "red2"),
  legendTextSize = 10,
  legendTitleSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  theme = c("minimal", "bw", "classic", "linedraw"),
  ...
)
}
\arguments{
\item{df}{A data frame with rownames as genes, a log column and a
p-value column.}

\item{title}{Plot title.}

\item{logCol}{Log column.}

\item{pvalCol}{P-value column.}

\item{xLab}{x axis label.}

\item{yLab}{y axis label.}

\item{legendTitle}{Legend title.}

\item{legendLabs}{Legend labels.}

\item{legendPos}{Legend position.}

\item{logFCThr}{Threshold used to separate significant log values.}

\item{pvalThr}{Threshold used to separate significant p-values.}

\item{labeledGenes}{Gene labels to be displayed on the plot. Default is
\code{NULL}, entailing that gene labels will be displayed on the basis
of \code{labLogFCThr} and \code{labPvalThr}, if at least one of
them is not \code{NULL}.}

\item{labelOutside}{Display labels for points specified by
\code{labeledGenes} even if they fall outside the boundaries imposed by
\code{labLogFCThr} and \code{labPvalThr}. Ignored if \code{labeledGenes} is
\code{NULL} or both \code{labLogFCThr} and \code{labPvalThr} are \code{NULL}.}

\item{labLogFCThr}{Threshold used to plot gene labels based on log values.
Ignored if \code{labeledGenes} is not \code{NULL}.}

\item{labPvalThr}{Threshold used to plot gene labels based on p-values.
Ignored if \code{labeledGenes} is not \code{NULL}.}

\item{labelType}{Whether to draw a box around labels (option 'boxed') or not
(option 'free'). Default is 'free'.}

\item{labelSize}{Label size.}

\item{labelColor}{Label color.}

\item{labelRepulsion}{Repulsion strength between labels.}

\item{labelPull}{Attraction strength between a text label
and its data point.}

\item{maxOverlaps}{Maximum number of allowed overlaps.}

\item{boxPadding}{Amount of padding around box.}

\item{labelPadding}{Amount of padding around label.}

\item{labelSegWidth}{Thickness of segment connecting label to point.}

\item{pointSize}{Point size.}

\item{alpha}{Opaqueness level of point color.}

\item{palette}{Color palette.}

\item{legendTextSize}{Legend text size.}

\item{legendTitleSize}{Legend title size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{theme}{Plot theme. Choose between 'bw', 'classic', 'linedraw'
and 'minimal'. Default is 'minimal'.}

\item{...}{Additional arguments passed to
\code{EnhancedVolcano::EnhancedVolcano}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function creates a volcano plot for a data frame with a log column and
a p-value column. The gene names must be provided as row names.
}
\details{
Users can input labeled genes in two ways:

1) By using p-value and log fold-change thresholds (the default option).

2) By inputting a list of labels.
}
\examples{
if (requireNamespace("EnhancedVolcano", quietly=TRUE)){
filePath <- system.file('extdata', 'volcanoPlot.qs2', package='henna')
df <- qs2::qs_read(filePath)
p <- volcanoPlot(df, title='Volcano plot - beta cells', pvalThr=1e-10,
logFCThr=1,
labPvalThr=1e-150,
labLogFCThr=5.3)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/palettes.R
\name{dpColors}
\alias{dpColors}
\title{Create a palette designed for densityPlot}
\usage{
dpColors(
  palette = c("cake", "cloudy", "grapes", "lava", "oasis", "orichalc", "sea", "sky")
)
}
\arguments{
\item{palette}{One of 'lava', 'oasis', 'orichalc', 'sea' and 'sky'.}
}
\value{
A character vector of colors.
}
\description{
This function returns a palette designed for \code{densityPlot}.
}
\examples{
dpColors('sea')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation.R
\name{documentFun}
\alias{documentFun}
\title{Internal function used for documenting other functions}
\usage{
documentFun(
  title = NULL,
  xLab = "x",
  yLab = "y",
  legendTitle = "Legend",
  legendLabs = c("a", "b"),
  legendPos = "right",
  palette = "Spectral",
  viridisPal = "turbo",
  labeledPoints = NULL,
  labelType = "free",
  labelOutside = TRUE,
  labelSize = 2.5,
  labelColor = "black",
  labelRepulsion = 1,
  labelPull = 1,
  maxOverlaps = 50,
  boxPadding = 0.2,
  labelPadding = 0.1,
  labelSegWidth = 0.4,
  pointSize = 0.8,
  pointShape = 1,
  legendTitleSize = 10,
  legendTextSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  xAngle = 45,
  vJust = 1,
  hJust = 1,
  margins = margin(0, -10, -10, -10),
  theme = "linedraw",
  pvalOffset = 1.00000023069254e-317,
  ...
)
}
\arguments{
\item{title}{Plot title.}

\item{xLab}{x axis label.}

\item{yLab}{y axis label.}

\item{legendTitle}{Legend title.}

\item{legendLabs}{Legend labels.}

\item{legendPos}{Legend position.}

\item{palette}{Color palette.}

\item{viridisPal}{Viridis palette.}

\item{labeledPoints}{Point labels to be displayed on the plot.}

\item{labelType}{Whether to draw a box around labels (option 'boxed') or not
(option 'free'). Default is 'free'.}

\item{labelSize}{Label size.}

\item{labelColor}{Label color.}

\item{labelRepulsion}{Repulsion strength between labels.}

\item{labelPull}{Attraction strength between a text label
and its data point.}

\item{maxOverlaps}{Maximum number of allowed overlaps.}

\item{boxPadding}{Amount of padding around box.}

\item{labelPadding}{Amount of padding around label.}

\item{labelSegWidth}{Thickness of segment connecting label to point.}

\item{pointSize}{Point size.}

\item{pointShape}{Point shape.}

\item{legendTitleSize}{Legend title size.}

\item{legendTextSize}{Legend text size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{xAngle}{Angle of x axis text.}

\item{vJust}{Vertical justification in [0, 1].}

\item{hJust}{Horizontal justification in [0, 1].}

\item{margins}{Plot margins. Must be a vector of size 4 listing the desired
top, right, bottom and left margin, in that order.}

\item{theme}{Plot theme.}

\item{pvalOffset}{Offset added to p-values to avoid infinite values when
taking logarithms.}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
\code{NULL}. This function is only used internally for
documentation.
}
\description{
This function is used internally to help document other functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/density_plot.R
\name{densityPlot}
\alias{densityPlot}
\title{Create density plot}
\usage{
densityPlot(
  df,
  title = NULL,
  colorScheme = c("cloudy", "cake", "grapes", "lava", "oasis", "orichalc", "sea", "sky",
    "custom"),
  useSchemeDefaults = TRUE,
  drawNN = TRUE,
  drawScores = FALSE,
  xLab = NULL,
  yLab = NULL,
  legendTitle = "Density",
  palette = NULL,
  segColor = "black",
  pointSize = 1,
  pointColor = "red",
  segType = c("dashed", "solid", "dotted", "dotdash", "longdash", "twodash"),
  segWidth = 0.4,
  nGridPoints = 300,
  expandPerc = 20,
  labelType = c("free", "boxed"),
  labelSize = 3,
  labelColor = "black",
  labelRepulsion = 1,
  labelPull = 1,
  maxOverlaps = 10,
  labelPadding = 0,
  boxPadding = 0,
  labelSegWidth = 0.4,
  legendPos = c("right", "none"),
  legendTextSize = 10,
  legendTitleSize = 10,
  axisTextSize = 12,
  axisTitleSize = 12,
  verbose = FALSE,
  ...
)
}
\arguments{
\item{df}{A data frame with at least two columns, representing the \code{x}
and \code{y} coordinates of the points. A score column can also be provided
as the third column. Nearest neighbor information can be provided in the last
column as a character vector with elements selected from the rownames.}

\item{title}{Plot title.}

\item{colorScheme}{Color scheme. Choose between 'cake', 'cloudy', 'grapes',
'lava', oasis', 'orichalc', 'sea', 'sky' and 'custom'. Default is 'cloudy'.}

\item{useSchemeDefaults}{Whether to use the default \code{segColor},
\code{pointColor} and \code{labelColor} values for scheme. Ignored
if \code{colorScheme} is set to 'custom'.}

\item{drawNN}{Whether to draw segments linking each point to its nearest
neighbor.}

\item{drawScores}{Whether to render scores on the plot. If set to
\code{TRUE}, the third column of the input data frame will be numeric and
scores will be taken from there.}

\item{xLab}{x axis label.}

\item{yLab}{y axis label.}

\item{legendTitle}{Legend title.}

\item{palette}{A character vector of colors. Used only if color scheme is
set to 'custom'.}

\item{segColor}{Nearest neighbor segment color. Ignored if \code{drawNN} is
set to \code{FALSE}, or if \code{useSchemeDefaults} is \code{TRUE} and
\code{colorScheme} is different from 'custom'.}

\item{pointSize}{Point size.}

\item{pointColor}{Point color. Ignored if \code{useSchemeDefaults}
is \code{TRUE} and \code{colorScheme} is different from 'custom'.}

\item{segType}{Nearest neighbor segment type. Choose between 'solid',
'dashed', 'dotted','dotdash', 'longdash' and 'twodash'. Ignored if
\code{drawNN} is set to \code{FALSE}.}

\item{segWidth}{Nearest neighbor segment width. Ignored if \code{drawNN} is
set to \code{FALSE}.}

\item{nGridPoints}{Number of grid points in each direction.}

\item{expandPerc}{Percentage by which the grid will be expanded.}

\item{labelType}{Whether to draw a box around labels (option 'boxed') or not
(option 'free'). Default is 'free'.}

\item{labelSize}{Label size.}

\item{labelColor}{Label color. Ignored if \code{useSchemeDefaults}
is \code{TRUE} and \code{colorScheme} is different from 'custom'.}

\item{labelRepulsion}{Repulsion strength between labels.}

\item{labelPull}{Attraction strength between a text label
and its data point.}

\item{maxOverlaps}{Maximum number of allowed overlaps.}

\item{labelPadding}{Amount of padding around label.}

\item{boxPadding}{Amount of padding around box.}

\item{labelSegWidth}{Thickness of segment connecting label to point.}

\item{legendPos}{Legend position. Choose between 'right' and 'none'.}

\item{legendTextSize}{Legend text size.}

\item{legendTitleSize}{Legend title size.}

\item{axisTextSize}{Axis text size.}

\item{axisTitleSize}{Axis title size.}

\item{verbose}{Whether output should be verbose.}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function creates a density plot.
}
\examples{
x <- c(1, 2, 3, 4, 6, 7, 8, 10, 12, 11, 3, 6, 4, 1, 13, 13, 14, 18, 16)
y <- c(1, 3, 1, 4, 3, 2, 8, 2, 1, 11, 8, 8, 10, 14, 13, 11, 11, 12,15)
z <- round(runif(19, 75, 100), 2)
df <- data.frame(x, y, z)
rownames(df) <- paste0('p', rownames(df))
densityPlot(df)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{isPointOnSeg}
\alias{isPointOnSeg}
\title{Check if a point is on a segment}
\usage{
isPointOnSeg(xPoint, yPoint, xStart, yStart, xEnd, yEnd)
}
\arguments{
\item{xPoint}{x coordinate of point P.}

\item{yPoint}{y coordinate of point P.}

\item{xStart}{x coordinate of point A.}

\item{yStart}{y coordinate of point A.}

\item{xEnd}{x coordinate of point B.}

\item{yEnd}{y coordinate of point B.}
}
\value{
Logical; whether the point is on the segment.
}
\description{
This function checks if a point P is on a segment AB.
}
\examples{
isPointOnSeg(2, 3, 1, 2, 3, 4)
isPointOnSeg(2, 3, 1, 2, 3, 8)
isPointOnSeg(4, 5, 1, 2, 3, 4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{convexHull}
\alias{convexHull}
\title{Construct the convex hull of a set of points}
\usage{
convexHull(pointsDF, hullIndices = NULL)
}
\arguments{
\item{pointsDF}{A data frame with the x and y coordinates of the points.}

\item{hullIndices}{Precalculated hull indices. Default is \code{NULL}: hull
indices are not provided, but they are calculated by \code{convexHull}.}
}
\value{
A data frame with two columns representing the points on the convex
hull.
}
\description{
This function constructs the convex hull of a set of points.
}
\details{
The points must be provided as a data frame with two columns.
}
\examples{
pointsDF <- data.frame(a = c(1, 2, 2, 3, 3, 4, 5, 6, 8, 6,
7, 8, 6, 8, 10, 3, 1),
b = c(2, 3, 4, 8, 5, 6, 5, 4, 8, 11, 13, 14, 2, 1, 2, 14, 9))
hull <- convexHull(pointsDF)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/radial_plot.R
\name{radialPlot}
\alias{radialPlot}
\title{Draw radial plot for a data frame with positive integer-valued points}
\usage{
radialPlot(
  valuesDF,
  title = NULL,
  valueLegendTitle = "Value",
  groupLegendTitle = NULL,
  extraCircles = 0,
  palette = rpColors(length(unique(valuesDF[, 3]))),
  labelSize = 3,
  pointSize = 0.8,
  legendTitleSize = 10,
  legendTextSize = 10,
  labelRepulsion = 1,
  labelPull = 0,
  maxOverlaps = 15,
  breakDensity = 6,
  seed = 50,
  ...
)
}
\arguments{
\item{valuesDF}{A data frame with names on the first column and
positive integers on the second column.}

\item{title}{Plot title.}

\item{valueLegendTitle}{Legend title corresponding to the positive integer
column.}

\item{groupLegendTitle}{Legend title corresponding to the categorical
column.}

\item{extraCircles}{Number of circles drawn beyond those required to include
the points.}

\item{palette}{Color palette.}

\item{labelSize}{Label size.}

\item{pointSize}{Point size.}

\item{legendTitleSize}{Legend title size.}

\item{legendTextSize}{Legend text size.}

\item{labelRepulsion}{Repulsion strength between labels.}

\item{labelPull}{Attraction strength between a text label
and its data point.}

\item{maxOverlaps}{Maximum number of allowed overlaps.}

\item{breakDensity}{Factor used in calculating the number of breaks for
the values legend. Higher values of this argument add more breaks to the
legend, but no breaks at a distance below 1 will be allowed.}

\item{seed}{Random seed.}

\item{...}{Additional arguments passed to \code{centerTitle}.}
}
\value{
An object of class \code{gg}.
}
\description{
This function draws a radial plot for a data frame, plotting positive
integer-valued points over concentric circles, with points located
more centrally representing higher values.
}
\examples{
valuesDF <- data.frame(Protein = paste0('P', seq(20)),
Value = sample(10, 20, replace=TRUE),
Group = sample(3, 20, replace=TRUE))
radialPlot(valuesDF, groupLegendTitle='Group')

}
